/***************************************************************************
 *            confutils.c
 *
 *  Wed Aug  4 18:50:13 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#define MAX_LINE_LENGTH 350

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "confutils.h"

// repositions ConfFile to the line after "val" = , assumedly after GotoConfSection
// has been called.  optionally copies said line into char* line
gboolean GotoConfVal(const char* val, FILE* ConfFile, char* line);

// repositions ConfFile to the line after [section]
gboolean GotoConfSection(const char* section, FILE* ConfFile);

gboolean GotoConfVal(const char* val, FILE* ConfFile, char* line)
{
	gboolean freeline = FALSE;
	if(line == NULL){
		line = malloc(MAX_LINE_LENGTH);
		freeline = TRUE;
	}

	long origpos = ftell(ConfFile);
	
	while(fgets(line, MAX_LINE_LENGTH, ConfFile) != NULL) {
	
		if(strncmp(line, "[", 1) == 0)
			break;
		
		// searches for the value
		if(strncmp(line + 1, val, strlen(val)) == 0
			&& line[strlen(val) + 1] == '"'
			&& strncmp(line, ";", 1) != 0
			&& strncmp(line, "\n", 1) != 0)
		{
			if(freeline)
				free(line);
			return TRUE;
		}	
	}
	
	if(freeline)
		free(line);
	
	fseek(ConfFile, origpos, SEEK_SET);
	return FALSE;
}

gboolean GotoConfSection(const char* section, FILE* ConfFile)
{
	char line[MAX_LINE_LENGTH];
	
	rewind(ConfFile);
	while(!feof(ConfFile))
	{
		fgets(line, MAX_LINE_LENGTH, ConfFile);
		if( strncmp(line + 1, section, strlen(section)) == 0
			&& ( *(line + 1 + strlen(section)) == ']' || *(line + 1 + strlen(section)) == '\\' ) )
		{
			return TRUE;
		}
	}
	
	rewind(ConfFile);
	return FALSE;
}

char* conf_GetVal(const char* val, const char* section, FILE* ConfFile)
{
	char* line;
	char* response;
	
	if( !GotoConfSection(section, ConfFile) )
		return NULL;
			
	line = malloc(MAX_LINE_LENGTH);
	if( !GotoConfVal(val, ConfFile, line) )
		return NULL;
	
	char* tmp = strchr(line, '=');
	tmp = strchr(tmp, '"') + 1;
	*strchr(tmp, '"') = '\0';
	response = malloc(strlen(tmp) + 1);
	strcpy(response, tmp);
	
	return response;
}

char* conf_GetCustomVal(const char* val, const char* section, const char* executable, FILE* ConfFile)
{
	char appdefault[13 + strlen(executable) + 2 + strlen(section) + 1];
	sprintf(appdefault, "AppDefaults\\\\%s\\\\%s", executable, section);
	
	return conf_GetVal(val, appdefault, ConfFile);
}

char* conf_GetGameVal(const char* val, const char* section, const char* executable, FILE* ConfFile)
{
	char* tmp = conf_GetCustomVal(val, section, executable, ConfFile);
	if(tmp == NULL)
		tmp = conf_GetVal(val, section, ConfFile);
	
	return tmp;
}

void conf_RemoveVal(const char* val, const char* section)
{
	char filename[strlen(getenv("HOME")) + 20];
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
	
	FILE* ConfFile = fopen(filename, "r+");
	if(ConfFile == NULL){
		printf("Could not open config file: %s\n", filename);
		return;
	}
	
	// go to the line after the section
	if( !GotoConfSection(section, ConfFile) )
		return;
	
	// go to the line after the value
	if( !GotoConfVal(val, ConfFile, NULL) )
		return;
	
	// set up startbuf and endbuf so we don't segfault when calling strlen() below
	char* line = malloc(MAX_LINE_LENGTH);
	char* endbuf = malloc(1);
	char* startbuf = malloc(1);
	endbuf[0] = '\0';
	startbuf[0] = '\0';
	
	// linepos is the line after the value.  start copying from there into endbuf
	long linepos = ftell(ConfFile);
	while(fgets(line, MAX_LINE_LENGTH, ConfFile) != NULL){
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}	
	
	rewind(ConfFile);
	
	// this clever block of code copies from the beginning into startbuf, then stops
	// immediately after it reads our line, so the line never gets copied into startbuf
	fgets(line, MAX_LINE_LENGTH, ConfFile);
	while(ftell(ConfFile) != linepos){
		startbuf = realloc(startbuf, strlen(startbuf) + strlen(line) + 1);
		strcat(startbuf, line);
		fgets(line, MAX_LINE_LENGTH, ConfFile);
	}
	
	// truncate the file to 0 and write both buffers in
	freopen(filename, "w", ConfFile);
	fwrite(startbuf, 1, strlen(startbuf), ConfFile);
	fwrite(endbuf, 1, strlen(endbuf), ConfFile);
	
	// flush the buffer
	fclose(ConfFile);
}

void conf_RemoveCustomVal(const char* val, const char* section, const char* executable)
{
	char filename[strlen(getenv("HOME")) + 20];
	char appdefault[13 + strlen(executable) + 2 + strlen(section) + 1];
	
	sprintf(appdefault, "AppDefaults\\\\%s\\\\%s", executable, section);
	conf_RemoveVal(val, appdefault);
	
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
	FILE* config = fopen(filename, "r+");
	if(config == NULL)
		return;
	
	if(!GotoConfSection(appdefault, config)){
		fclose(config);
		return;
	}
	
	long linepos = ftell(config);
	
	char* line = malloc(500);
	char* endbuf = malloc(1);
	endbuf[0] = '\0';
	
	while(fgets(line, 500, config) != NULL){
		if(line[0] == '[')
			break;
		if(line[0] == '"'){
			fclose(config);
			free(line);
			free(endbuf);
			return;
		}
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	
	if(line != NULL){
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	
	while(fgets(line, 500, config) != NULL){
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	
	char* startbuf = malloc(1);
	startbuf[0] = '\0';
	
	rewind(config);
	fgets(line, 500, config);
	while(ftell(config) != linepos){
		startbuf = realloc(startbuf, strlen(startbuf) + strlen(line) + 1);
		strcat(startbuf, line);
		fgets(line, 500, config);
	}
	
	freopen(filename, "w", config);
	fwrite(startbuf, 1, strlen(startbuf), config);
	fwrite(endbuf, 1, strlen(endbuf), config);
	fclose(config);
	
	free(startbuf);
	free(endbuf);
	free(line);
	
}

void conf_SetGameVal(
	const char* newval, const char* val,
	const char* section, const char* executable)
{
	if(newval == NULL){
		conf_RemoveCustomVal(val, section, executable);
		return;
	}
	
	char filename[strlen(getenv("HOME")) + 20];
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
		
	FILE* ConfFile = fopen(filename, "r+");
	if(ConfFile == NULL){
		printf("Could not open config file for writing: %s", filename);
		return;
	}
	
	// if newval == oldval, return now
	char* oldval = conf_GetGameVal(val, section, executable, ConfFile);
	if(oldval != NULL){
		if(strcmp(oldval, newval) == 0){
			free(oldval);
			fclose(ConfFile);
			return;
		}
		free(oldval);
	}
	
	// if newval == defval, remove the custom value
	char* defval = conf_GetVal(val, section, ConfFile);
	if(defval != NULL){
		if(strcmp(newval, defval) == 0){
			fclose(ConfFile);
			conf_RemoveCustomVal(val, section, executable);
			free(defval);
			return;
		}
		free(defval);
	}
	long  linepos;
	char* line;
	// create the custom section for our value
	char* appdefault = malloc(13 + strlen(executable) + 2 + strlen(section) + 1);
	sprintf(appdefault, "AppDefaults\\\\%s\\\\%s", executable, section);
	
	// if the section doesn't exist . . .
	if(!GotoConfSection(appdefault, ConfFile)){
		
		// try again for [AppDefaults\\executable
		appdefault[ 13 + strlen(executable) ] = '\0';
		if(GotoConfSection(appdefault, ConfFile)) {
			
			// change appdefault back
			appdefault[ 13 + strlen(executable) ] = '\\';
			
			// make linepos immediately before the next section after the first one found (or a newline)
			line = malloc(MAX_LINE_LENGTH);
			while( 
				! (line[0] == '[' || line[0] == '\n')
				&& !feof(ConfFile) )
			{
				linepos = ftell(ConfFile);
				fgets(line, MAX_LINE_LENGTH, ConfFile);
			}
			
			// copy everything after linepos into endbuf
			char* endbuf = malloc(strlen(line) + 1);
			strcpy(endbuf, line);
		
			fgets(line, MAX_LINE_LENGTH, ConfFile);
			while(!feof(ConfFile)){
				endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
				strcat(endbuf, line);
				fgets(line, MAX_LINE_LENGTH, ConfFile);
			}
			
			// go back to linepos, output the line for the new section, then append endbuf
			fseek(ConfFile, linepos, SEEK_SET);
			
			fprintf(ConfFile, "[%s]\n", appdefault);
			fwrite(endbuf, 1, strlen(endbuf), ConfFile);
			
			free(endbuf);
			free(line);
		
		} else { // if the section doesn't exist at all, just let conf_SetVal handle it
			// change appdefault back
			appdefault[ 13 + strlen(executable) ] = '\\';
			
		} // if GotoConfSection appdefault//executable
	} // if GotoConfSection appdefault//executable/section
	fclose(ConfFile);
	
	conf_SetVal(newval, val, appdefault);
	free(appdefault);
}

void conf_SetVal(const char* newval, const char* val, const char* section)
{
	if(newval == NULL){
		conf_RemoveVal(val, section);
		return;
	}
	
	char filename[strlen(getenv("HOME")) + 20];
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
		
	FILE* ConfFile = fopen(filename, "r+");
	if(ConfFile == NULL){
		printf("Could not open config file for writing: %s", filename);
		return;
	}
	
	// if newval == oldval, return now
	char* oldval = conf_GetVal(val, section, ConfFile);
	if(oldval != NULL){
		if(strcmp(oldval, newval) == 0){
			free(oldval);
			fclose(ConfFile);
			return;
		}
		free(oldval);
	}
	
	long  linepos;
	char* line;
	
	// if the section doesn't exist, create it
	if(!GotoConfSection(section, ConfFile)) {
		
		line = malloc(MAX_LINE_LENGTH);
		line[0] = '\0';
		
		// linepos is set immediately before the # </wineconf> line
		while( !(line[0] == '#' && strstr(line, "</wineconf>")) )
		{
			linepos = ftell(ConfFile);
			if(fgets(line, 500, ConfFile) == NULL){
				printf("Error: ./transgaming/config does not end with a # </wineconf> line\n");
				free(line);
				fclose(ConfFile);
				return;
			}
		}
		
		// go back to the immediate line before # </wineconf> and output our section, then
		// output the # </wineconf> line since we just rewrote it
		fseek(ConfFile, linepos, SEEK_SET);
		
		fprintf(ConfFile, "\n[%s]\n\n", section);
		fputs(line, ConfFile);
		
		free(line);
		
		GotoConfSection(section, ConfFile);
	}
	
	// what's below is a workaround for potentially having to call strlen() on a null pointer below
	char* endbuf = malloc(1);
	char* startbuf = malloc(1);
	endbuf[0] = '\0';
	startbuf[0] = '\0';
	line = malloc(MAX_LINE_LENGTH);
	
	// position the stream immediately after the line if it exists
	gboolean exists;
	if(GotoConfVal(val, ConfFile, NULL)) {
		linepos = ftell(ConfFile);
		fgets(line, MAX_LINE_LENGTH, ConfFile);
		exists = TRUE;
	} else {
		linepos = ftell(ConfFile);
		exists = FALSE;
	}
	
	// copy from linepos to eof into endbuf
	while(fgets(line, MAX_LINE_LENGTH, ConfFile) != NULL){
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	
	// copy from beginning of file to line before linepos into startbuf
	rewind(ConfFile);
	while(ftell(ConfFile) != linepos){
		fgets(line, MAX_LINE_LENGTH, ConfFile);
		startbuf = realloc(startbuf, strlen(startbuf) + strlen(line) + 1);
		strcat(startbuf, line);
	}
	
	// this should trim out the line if it already exists
	if(exists) {
		char* tmp = strrchr(startbuf, '\n');
		*tmp = '\0';
		tmp = strrchr(startbuf, '\n');
		*(tmp + 1) = '\0';
	}
	
	// truncate the file to 0 and write both buffers with the line in between
	freopen(filename, "w", ConfFile);
	
	fwrite(startbuf, 1, strlen(startbuf), ConfFile);
	fprintf(ConfFile, "\"%s\" = \"%s\"\n", val, newval);
	fwrite(endbuf, 1, strlen(endbuf), ConfFile);
	
	// we're done.  use fclose to flush the buffer.
	
	free(startbuf);
	free(endbuf);
	free(line);
	fclose(ConfFile);
}
