/***************************************************************************
 *            desktop_entry.c
 *
 *  Sun Aug  8 17:18:12 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MAX_VALUE_LENGTH 500

char* desktop_entry_GetValue(const char* name, const char* path)
{
	FILE* file = fopen(path, "r");
	if(file == NULL){
		printf("Error opening desktop entry: %s\n", path);
		return NULL;
	}
	
	char* line = malloc(MAX_VALUE_LENGTH);
	
	// check to see if the first line is "[Desktop Entry]"
	// incidentally Gnome and KDE shortcuts are the exact same format
	fgets(line, MAX_VALUE_LENGTH, file);
	if( strcmp(line, "[Desktop Entry]\n") != 0 ){
		printf("Not a valid desktop entry: %s\n", path);
		fclose(file);
		free(line);
		return NULL;
	}
	
	fgets(line, MAX_VALUE_LENGTH, file);
	while(!feof(file))
	{
		if( strncmp(line, name, strlen(name)) == 0 )
		{	 
			// copy after the "=", replace \n with \0
			char* tmp = strchr(line, '=') + 1;
			tmp[strlen(tmp) - 1] = '\0';
			
			char* response = malloc(strlen(tmp) + 1);
			strcpy(response, tmp);
			
			fclose(file);
			free(line);
			return response;
		}
		fgets(line, MAX_VALUE_LENGTH, file);
	}
	
	fclose(file);
	free(line);
	return NULL;
}

void desktop_entry_SetValue(const char* val, const char* name, const char* path)
{		
	FILE* file;
	char* existing = desktop_entry_GetValue(name, path);
	// if the value doesn't exist, append it to the file and leave
	if(existing == NULL){
		file = fopen(path, "a");
		fprintf(file, "%s=%s\n", name, val);
		fclose(file);
		return;
	}
	
	// if the requested change is the same as the existing, do nothing.
	if( strcmp(val, existing) == 0 ){
		free(existing);
		return;
	}
	free(existing);
	
	// we know path exists from GetValue so we don't double-check
	file = fopen(path, "r+");
	
	char* line = malloc(500);
	// this is so we don't call strlen() on a null pointer below
	char* startbuf = malloc(1);
	char* endbuf = malloc(1);
	startbuf[0] = '\0';
	endbuf[0] = '\0';
	
	// feof is not included in the condition because we know the value exists
	while(fgets(line, 500, file) != NULL ) {
		if( !(
		 	strncmp( line, name, strlen(name) ) == 0 
		 	&& line[strlen(name)] == '='
			)
		){	
			startbuf = realloc(startbuf, strlen(startbuf) + strlen(line) + 1);
			strcat(startbuf, line);
		} else
			break;
	}
	
	// note that we have conveniently skipped over the existing line
	while(fgets(line, 500, file) != NULL)
	{
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	free(line);
	
	// rewrite the file with the new line between startbuf and endbuf
	freopen(path, "w", file);
	
	fwrite(startbuf, 1, strlen(startbuf), file);
	fprintf(file, "%s=%s\n", name, val);
	fwrite(endbuf, 1, strlen(endbuf), file);
	
	free(startbuf);
	free(endbuf);
	fclose(file);
}
