/***************************************************************************
 *            favorites.c
 *
 *  Sun Aug  8 17:04:06 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include <gtk/gtk.h>

#include "favorites.h"
#include "desktop_entry.h"

void favorites_TreeViewFill(GtkTreeView* FavoritesView)
{
	char *path;
	path = malloc(strlen(getenv("HOME")) + 21 + 1);
	sprintf(path, "%s/.grapevine/favorites", getenv("HOME"));
	
	DIR* Dir = opendir(path);
	if(Dir == NULL){
		printf("Error opening directory: %s\n", path);
		return;
	}
	
	GtkListStore* FavoritesStore = gtk_list_store_new(
		NUM_COLUMNS,
		GDK_TYPE_PIXBUF,  // ICON_COLUMN
		G_TYPE_STRING,    // NAME_COLUMN
		G_TYPE_STRING,    // PATH_COLUMN
		G_TYPE_INT        // TYPE_COLUMN
	);
	
	struct dirent* entry;

	do{
		entry = readdir(Dir);
		if(entry==NULL) break;
	}while(entry->d_name[0] == '.');
	
	while(entry != NULL) {
		
		// build the path for this directory
		char* shortcutpath = malloc(strlen(path) + 1 + strlen(entry->d_name) + 1);
		sprintf(shortcutpath, "%s/%s", path, entry->d_name);
		
		// some information on the file will be stored in this
		struct stat fileinfo;
		
		stat(shortcutpath, &fileinfo);
				
		if(S_ISREG(fileinfo.st_mode)
			&& strcmp(strrchr(entry->d_name, '.'), ".desktop") == 0 )
		{
			char* shortcutpath = malloc(strlen(path) + 1 + strlen(entry->d_name) + 1);
			sprintf(shortcutpath, "%s/%s", path, entry->d_name);
			char* name     = desktop_entry_GetValue("Name", shortcutpath);
			char* iconpath = desktop_entry_GetValue("Icon", shortcutpath);
			
			if(name != NULL)
			{
				GtkTreeIter iter;
				gtk_list_store_append(FavoritesStore, &iter);
				
				if(iconpath != NULL) {
					GdkPixbuf* icon;
					GError* pixbuferror = NULL;
					icon = gdk_pixbuf_new_from_file(iconpath, &pixbuferror);
					
					if(icon == NULL)
						printf("Error loading pixbuf: %s\n", pixbuferror->message);
					else {
						// enter the name and icon
							gtk_list_store_set(
							FavoritesStore, &iter,
							ICON_COLUMN, icon,
							-1);
					}
				
				gtk_list_store_set(
					FavoritesStore, &iter,
					NAME_COLUMN, name,
					PATH_COLUMN, shortcutpath,
					TYPE_COLUMN, SHORTCUT_TYPE,
					-1);
					
				} // if iconpath != NULL
			} // if name != NULL
			
			free(shortcutpath);
			free(name);
			free(iconpath);
			
		} // if entry->d_type == DT_REG
		
		entry = readdir(Dir);
	} // while entry != NULL
	
	closedir(Dir);
	free(path);
	gtk_tree_view_set_model(FavoritesView, GTK_TREE_MODEL(FavoritesStore));
}
