/***************************************************************************
 *            gameconfdialog.c
 *
 *  Fri Jul  9 14:00:44 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>

#include <string.h>

#include "wineconfig.h"
#include "wineexecutils.h"

extern GladeXML* gameconfdialog;
	
void gameconf_runGameConfDialog(const char* executable)
{
	// get the game's configuration
	WineConfig _conf;
	WineConfig* conf = &_conf;
	init_WineConfig(conf);
	if( !wineconfig_GetGameConfig(conf, executable) )
		return;
	
	// get all of our widgets
	GtkWidget* GameConfDialog = glade_xml_get_widget(gameconfdialog, "GameConfDialog");
	GtkWidget* WinverBox = glade_xml_get_widget(gameconfdialog, "WinverBox");
	GtkWidget* DesktopEntry = glade_xml_get_widget(gameconfdialog, "DesktopEntry");
	GtkWidget* ManagedButton = glade_xml_get_widget(gameconfdialog, "ManagedButton");
	GtkWidget* DXGrabButton = glade_xml_get_widget(gameconfdialog, "DXGrabButton");
	GtkWidget* XVidModeButton = glade_xml_get_widget(gameconfdialog, "XVidModeButton");
	GtkWidget* XRandRButton = glade_xml_get_widget(gameconfdialog, "XRandRButton");
	GtkWidget* NVVARButton = glade_xml_get_widget(gameconfdialog, "NVVARButton");
	GtkWidget* ARBVBOButton = glade_xml_get_widget(gameconfdialog, "ARBVBOButton");
	GtkWidget* AnisoButton = glade_xml_get_widget(gameconfdialog, "AnisoButton");
	GtkWidget* VShaderButton = glade_xml_get_widget(gameconfdialog, "VShaderButton");
	GtkWidget* PShaderButton = glade_xml_get_widget(gameconfdialog, "PShaderButton");
	GtkWidget* ALSAButton = glade_xml_get_widget(gameconfdialog, "ALSAButton");
	GtkWidget* OSSButton = glade_xml_get_widget(gameconfdialog, "OSSButton");
	GtkWidget* MMapButton = glade_xml_get_widget(gameconfdialog, "MMapButton");
	GtkWidget* FullDuplexButton = glade_xml_get_widget(gameconfdialog, "FullDuplexButton");
	GtkWidget* SHMButton = glade_xml_get_widget(gameconfdialog, "SHMButton");
	GtkWidget* ReniceButton = glade_xml_get_widget(gameconfdialog, "ReniceButton");
	GtkWidget* BigExeButton = glade_xml_get_widget(gameconfdialog, "BigExeButton");
	GtkWidget* WorkDirButton = glade_xml_get_widget(gameconfdialog, "WorkDirButton");
	GtkWidget* WorkDirEntry = glade_xml_get_widget(gameconfdialog, "WorkDirEntry");
	GtkWidget* CmdLineButton = glade_xml_get_widget(gameconfdialog, "CmdLineButton");
	GtkWidget* CmdLineEntry = glade_xml_get_widget(gameconfdialog, "CmdLineEntry");
	GtkWidget* PthreadsBox = glade_xml_get_widget(gameconfdialog, "PthreadsBox");
	GtkWidget* WinexverBox = glade_xml_get_widget(gameconfdialog, "WinexverBox");
	GtkWidget* ClipSpaceFixButton = glade_xml_get_widget(gameconfdialog, "ClipSpaceFixButton");
	
	// set all of the widgets' initial states according to the values in conf
	//
	gtk_combo_box_set_active(GTK_COMBO_BOX(WinverBox), conf->vars[WINVER_VAR]);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(WinexverBox), WineExecIndex(conf->text[WINEEXEC_TEXT]) );
	
	if(conf->text[DESKTOP_TEXT] != NULL)
		gtk_entry_set_text(GTK_ENTRY (GTK_BIN (DesktopEntry)->child), conf->text[DESKTOP_TEXT]);
	else
		gtk_entry_set_text(GTK_ENTRY (GTK_BIN (DesktopEntry)->child), "None");

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ManagedButton), conf->vars[MANAGED_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ClipSpaceFixButton), conf->vars[CLIPSPACEFIX_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(DXGrabButton), conf->vars[DXGRAB_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(XVidModeButton), conf->vars[XVIDMODE_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(XRandRButton), conf->vars[XRANDR_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(NVVARButton), conf->vars[NVVAR_VAR]);	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ARBVBOButton), conf->vars[ARBVBO_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(AnisoButton), conf->vars[ANISO_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(VShaderButton), conf->vars[VSHADERS_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(PShaderButton), conf->vars[PSHADERS_VAR]);
	
	if(conf->vars[SOUNDDRV_VAR] == DRV_WINEALSA){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ALSAButton), TRUE);
		gtk_widget_set_sensitive(FullDuplexButton, FALSE);
	} else if(conf->vars[SOUNDDRV_VAR] == DRV_WINEOSS){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(OSSButton), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(FullDuplexButton), conf->vars[FDUPLEX_VAR]);
		gtk_widget_set_sensitive(FullDuplexButton, TRUE);
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(MMapButton), conf->vars[MMAP_VAR]);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SHMButton), conf->vars[SHMWINESERVER_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ReniceButton), conf->vars[RENICEWINESERVER_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(BigExeButton), conf->vars[BIGEXE_VAR]);
	
	if(conf->text[WORKDIR_TEXT] != NULL){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(WorkDirButton), TRUE);
		gtk_entry_set_text(GTK_ENTRY(WorkDirEntry), conf->text[WORKDIR_TEXT]);
		gtk_widget_set_sensitive(WorkDirEntry, TRUE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(WorkDirButton), FALSE);
		gtk_entry_set_text(GTK_ENTRY(WorkDirEntry), "");
		gtk_widget_set_sensitive(WorkDirEntry, FALSE);
	}
	
	if(conf->text[CMDLINE_TEXT] != NULL){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CmdLineButton), TRUE);
		gtk_entry_set_text(GTK_ENTRY(CmdLineEntry), conf->text[CMDLINE_TEXT]);
		gtk_widget_set_sensitive(CmdLineEntry, TRUE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CmdLineButton), FALSE);
		gtk_entry_set_text(GTK_ENTRY(CmdLineEntry), "");
		gtk_widget_set_sensitive(CmdLineEntry, FALSE);
	}
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(PthreadsBox), conf->vars[PTHREADS_VAR]);
	
	// this massive do-while statement keeps me from having to create a seperate function to
	// write the config so that the apply button will work
	gint response;
	do {
		response = gtk_dialog_run(GTK_DIALOG(GameConfDialog));
		
		if(response != GTK_RESPONSE_APPLY)
			gtk_widget_hide(GameConfDialog);
		if(response == GTK_RESPONSE_CANCEL){
			free_WineConfig(conf);
			return;
		}
		
		// set all WineConfig variables
		set_WineConfig_text(
			gtk_entry_get_text( GTK_ENTRY(GTK_BIN(DesktopEntry)->child) ),
			DESKTOP_TEXT,
			conf);
		
		conf->vars[WINVER_VAR]   = gtk_combo_box_get_active(GTK_COMBO_BOX(WinverBox));		
		
		set_WineConfig_text(
			WineExecText( gtk_combo_box_get_active(GTK_COMBO_BOX(WinexverBox)) ),
			WINEEXEC_TEXT,
			conf		
		);
		
		conf->vars[MANAGED_VAR]  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ManagedButton));
		conf->vars[CLIPSPACEFIX_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ClipSpaceFixButton));
		conf->vars[DXGRAB_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DXGrabButton));
		conf->vars[XVIDMODE_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(XVidModeButton));
		conf->vars[XRANDR_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(XRandRButton));
		conf->vars[NVVAR_VAR]    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(NVVARButton));
		conf->vars[ARBVBO_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ARBVBOButton));
		conf->vars[ANISO_VAR]    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AnisoButton));
		conf->vars[VSHADERS_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(VShaderButton));
		conf->vars[PSHADERS_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(PShaderButton));
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAButton)) )
			conf->vars[SOUNDDRV_VAR] = DRV_WINEALSA;
		else if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OSSButton)) ) {
			conf->vars[SOUNDDRV_VAR] = DRV_WINEOSS;
			conf->vars[FDUPLEX_VAR]  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(FullDuplexButton));
		}
		conf->vars[MMAP_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MMapButton));
		
		conf->vars[SHMWINESERVER_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SHMButton));
		conf->vars[RENICEWINESERVER_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ReniceButton));
		conf->vars[BIGEXE_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(BigExeButton));
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(WorkDirButton)) )
			set_WineConfig_text(gtk_entry_get_text(GTK_ENTRY(WorkDirEntry)), WORKDIR_TEXT, conf);
		else
			set_WineConfig_text(NULL, WORKDIR_TEXT, conf);
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CmdLineButton)) )
			set_WineConfig_text(gtk_entry_get_text(GTK_ENTRY(CmdLineEntry)), CMDLINE_TEXT, conf);
		else
			set_WineConfig_text(NULL, CMDLINE_TEXT, conf);
		
		conf->vars[PTHREADS_VAR] = gtk_combo_box_get_active(GTK_COMBO_BOX(PthreadsBox));
		
		// call the function which writes these to the config file
		wineconfig_SetGameConfig(conf, executable);
		
	} while(response == GTK_RESPONSE_APPLY);
	
	free_WineConfig(conf);
}

// Callback for ALSA/OSS radio buttons
void gameconf_snddrv_changed(void)
{
	GtkWidget* ALSAButton = glade_xml_get_widget(gameconfdialog, "ALSAButton");
	GtkWidget* FullDuplexButton = glade_xml_get_widget(gameconfdialog, "FullDuplexButton");
	
	gtk_widget_set_sensitive(
		FullDuplexButton,
		!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAButton)) );
}

// callback for WorkDirButton
void gameconf_WorkDirButton_toggled(GtkWidget* WorkDirButton)
{
	GtkWidget* WorkDirEntry = glade_xml_get_widget(gameconfdialog, "WorkDirEntry");
	gtk_widget_set_sensitive(
		WorkDirEntry,
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(WorkDirButton)) );
}

// callback for CmdLineButton
void gameconf_CmdLineButton_toggled(GtkWidget* CmdLineButton)
{
	GtkWidget* CmdLineEntry = glade_xml_get_widget(gameconfdialog, "CmdLineEntry");
	gtk_widget_set_sensitive(
		CmdLineEntry,
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CmdLineButton)) );
}

// callback for important note about setting Desktop to "None"
void gameconf_DesktopNote_clicked(GtkWidget* DesktopNoteButton)
{
	GtkWidget* DesktopNotePopup = glade_xml_get_widget(gameconfdialog, "DesktopNotePopup");
	
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DesktopNoteButton)) )
		gtk_widget_show(DesktopNotePopup);
	else
		gtk_widget_hide(DesktopNotePopup);
}

// callback to close note
void gameconf_DesktopNoteClose_clicked(void)
{
	GtkWidget* DesktopNotePopup = glade_xml_get_widget(gameconfdialog, "DesktopNotePopup");
	GtkWidget* DesktopNoteButton = glade_xml_get_widget(gameconfdialog, "DesktopNoteButton");
	
	gtk_widget_hide(DesktopNotePopup);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(DesktopNoteButton), FALSE);
}
