/***************************************************************************
 *            globalconfdialog.c
 *
 *  Fri Aug  6 15:04:21 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "wineconfig.h"
#include "wineexecutils.h"

extern GladeXML* globalconfdialog;

void globalconf_runGlobalConfDialog(void)
{
	WineConfig _conf;
	WineConfig* conf = &_conf;
	
	init_WineConfig(conf);
	if(!wineconfig_GetGlobalConfig(conf))
		return;
	
	// get all of our widgets
	GtkWidget* GlobalConfDialog = glade_xml_get_widget(globalconfdialog, "GlobalConfDialog");
	GtkWidget* WinverBox = glade_xml_get_widget(globalconfdialog, "WinverBox");
	GtkWidget* WinexverBox = glade_xml_get_widget(globalconfdialog, "WinexverBox");
	
	GtkWidget* DesktopEntry = glade_xml_get_widget(globalconfdialog, "DesktopEntry");
	GtkWidget* ManagedButton = glade_xml_get_widget(globalconfdialog, "ManagedButton");
	GtkWidget* ClipSpaceFixButton = glade_xml_get_widget(globalconfdialog, "ClipSpaceFixButton");
	GtkWidget* DXGrabButton = glade_xml_get_widget(globalconfdialog, "DXGrabButton");
	GtkWidget* XVidModeButton = glade_xml_get_widget(globalconfdialog, "XVidModeButton");
	GtkWidget* XRandRButton = glade_xml_get_widget(globalconfdialog, "XRandRButton");
	
	GtkWidget* VideoRamButton = glade_xml_get_widget(globalconfdialog, "VideoRamButton");
	GtkWidget* AGPMemButton = glade_xml_get_widget(globalconfdialog, "AGPMemButton");
	GtkWidget* NVVARButton = glade_xml_get_widget(globalconfdialog, "NVVARButton");
	GtkWidget* ARBVBOButton = glade_xml_get_widget(globalconfdialog, "ARBVBOButton");
	
	GtkWidget* AnisoButton = glade_xml_get_widget(globalconfdialog, "AnisoButton");
	GtkWidget* VShaderButton = glade_xml_get_widget(globalconfdialog, "VShaderButton");
	GtkWidget* PShaderButton = glade_xml_get_widget(globalconfdialog, "PShaderButton");
	
	GtkWidget* ALSAButton = glade_xml_get_widget(globalconfdialog, "ALSAButton");
	GtkWidget* OSSButton = glade_xml_get_widget(globalconfdialog, "OSSButton");
	GtkWidget* MMapButton = glade_xml_get_widget(globalconfdialog, "MMapButton");
	GtkWidget* FullDuplexButton = glade_xml_get_widget(globalconfdialog, "FullDuplexButton");
	
	GtkWidget* ALSAAdetectButton = glade_xml_get_widget(globalconfdialog, "ALSAAdetectButton");
	GtkWidget* OSSAdetectButton = glade_xml_get_widget(globalconfdialog, "OSSAdetectButton");
	GtkWidget* PCMLabel = glade_xml_get_widget(globalconfdialog, "PCMLabel");
	GtkWidget* CTLLabel = glade_xml_get_widget(globalconfdialog, "CTLLabel");
	GtkWidget* PCMEntry = glade_xml_get_widget(globalconfdialog, "PCMEntry");
	GtkWidget* CTLEntry = glade_xml_get_widget(globalconfdialog, "CTLEntry");
	GtkWidget* DSPLabel = glade_xml_get_widget(globalconfdialog, "DSPLabel");
	GtkWidget* DSPEntry = glade_xml_get_widget(globalconfdialog, "DSPEntry");
	GtkWidget* MixerLabel = glade_xml_get_widget(globalconfdialog, "MixerLabel");
	GtkWidget* MixerEntry = glade_xml_get_widget(globalconfdialog, "MixerEntry");
	
	GtkWidget* FreetypeButton = glade_xml_get_widget(globalconfdialog, "FreetypeButton");
	GtkWidget* SHMButton = glade_xml_get_widget(globalconfdialog, "SHMButton");
	GtkWidget* ReniceButton = glade_xml_get_widget(globalconfdialog, "ReniceButton");
	GtkWidget* PthreadsBox = glade_xml_get_widget(globalconfdialog, "PthreadsBox");
	
	// set all of the widgets' initial states according to the values in conf
	//
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(WinverBox), conf->vars[WINVER_VAR]);
	
	if(conf->text[DESKTOP_TEXT] != NULL)
		gtk_entry_set_text(GTK_ENTRY (GTK_BIN (DesktopEntry)->child), conf->text[DESKTOP_TEXT]);
	else
		gtk_entry_set_text(GTK_ENTRY (GTK_BIN (DesktopEntry)->child), "None");
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(WinexverBox), WineExecIndex(conf->text[WINEEXEC_TEXT]) );

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ManagedButton), conf->vars[MANAGED_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ClipSpaceFixButton), conf->vars[CLIPSPACEFIX_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(DXGrabButton), conf->vars[DXGRAB_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(XVidModeButton), conf->vars[XVIDMODE_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(XRandRButton), conf->vars[XRANDR_VAR]);
	
	if(conf->vars[VIDEORAM_VAR] != VAR_NULL)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(VideoRamButton), conf->vars[VIDEORAM_VAR]);
	if(conf->vars[AGPMEM_VAR] != VAR_NULL)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(AGPMemButton), conf->vars[AGPMEM_VAR]);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(NVVARButton), conf->vars[NVVAR_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ARBVBOButton), conf->vars[ARBVBO_VAR]);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(AnisoButton), conf->vars[ANISO_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(VShaderButton), conf->vars[VSHADERS_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(PShaderButton), conf->vars[PSHADERS_VAR]);
	
	if(conf->vars[SOUNDDRV_VAR] == DRV_WINEALSA){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ALSAButton), TRUE);
		gtk_widget_set_sensitive(FullDuplexButton, FALSE);
	} else if(conf->vars[SOUNDDRV_VAR] == DRV_WINEOSS){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(OSSButton), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(FullDuplexButton), conf->vars[FDUPLEX_VAR]);
		gtk_widget_set_sensitive(FullDuplexButton, TRUE);
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(MMapButton), conf->vars[MMAP_VAR]);
	
	if(conf->text[ALSA_DEV_TEXT] == NULL){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ALSAAdetectButton), TRUE);
		gtk_widget_set_sensitive(PCMLabel, FALSE);
		gtk_widget_set_sensitive(PCMEntry, FALSE);
		gtk_widget_set_sensitive(CTLLabel, FALSE);
		gtk_widget_set_sensitive(CTLEntry, FALSE);
	} else {
		gtk_entry_set_text(GTK_ENTRY(PCMEntry), conf->text[ALSA_DEV_TEXT]);
		gtk_entry_set_text(GTK_ENTRY(CTLEntry), conf->text[ALSA_CTL_TEXT]);
	}
	
	if(conf->text[OSS_DEV_TEXT] == NULL){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(OSSAdetectButton), TRUE);
		gtk_widget_set_sensitive(DSPLabel, FALSE);
		gtk_widget_set_sensitive(DSPEntry, FALSE);
		gtk_widget_set_sensitive(MixerLabel, FALSE);
		gtk_widget_set_sensitive(MixerEntry, FALSE);
	} else {
		gtk_entry_set_text(GTK_ENTRY(DSPEntry), conf->text[OSS_DEV_TEXT]);
		gtk_entry_set_text(GTK_ENTRY(MixerEntry), conf->text[OSS_MXR_TEXT]);
	}
	
	if(conf->vars[FREETYPE_VAR] != VAR_NULL)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(FreetypeButton), conf->vars[FREETYPE_VAR]);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SHMButton), conf->vars[SHMWINESERVER_VAR]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ReniceButton), conf->vars[RENICEWINESERVER_VAR]);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(PthreadsBox), conf->vars[PTHREADS_VAR]);
	
	// this massive do-while statement keeps me from having to create a seperate function to
	// write the config so that the apply button will work
	gint response;
	do {
		response = gtk_dialog_run(GTK_DIALOG(GlobalConfDialog));
		
		if(response != GTK_RESPONSE_APPLY)
			gtk_widget_hide(GlobalConfDialog);
		if(response == GTK_RESPONSE_CANCEL){
			free_WineConfig(conf);
			return;
		}
		
		set_WineConfig_text(
			gtk_entry_get_text( GTK_ENTRY(GTK_BIN(DesktopEntry)->child) ),
			DESKTOP_TEXT,
			conf
		);
		
		set_WineConfig_text(
			WineExecText( gtk_combo_box_get_active(GTK_COMBO_BOX(WinexverBox)) ),
			WINEEXEC_TEXT,
			conf
		);
		
		conf->vars[WINVER_VAR]   = gtk_combo_box_get_active(GTK_COMBO_BOX(WinverBox));
		conf->vars[MANAGED_VAR]  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ManagedButton));
		conf->vars[CLIPSPACEFIX_VAR]  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ClipSpaceFixButton));
		conf->vars[DXGRAB_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DXGrabButton));
		conf->vars[XVIDMODE_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(XVidModeButton));
		conf->vars[XRANDR_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(XRandRButton));
		
		conf->vars[VIDEORAM_VAR] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(VideoRamButton));
		conf->vars[AGPMEM_VAR]   = gtk_spin_button_get_value(GTK_SPIN_BUTTON(AGPMemButton));
		
		conf->vars[NVVAR_VAR]    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(NVVARButton));
		conf->vars[ARBVBO_VAR]   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ARBVBOButton));
		conf->vars[ANISO_VAR]    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AnisoButton));
		conf->vars[VSHADERS_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(VShaderButton));
		conf->vars[PSHADERS_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(PShaderButton));
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAButton)) )
			conf->vars[SOUNDDRV_VAR] = DRV_WINEALSA;
		else if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OSSButton)) ) {
			conf->vars[SOUNDDRV_VAR] = DRV_WINEOSS;
			conf->vars[FDUPLEX_VAR]  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(FullDuplexButton));
		}
		conf->vars[MMAP_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MMapButton));
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAAdetectButton)) ) {
			set_WineConfig_text("!AUTO", ALSA_DEV_TEXT, conf);
			set_WineConfig_text("!AUTO", ALSA_CTL_TEXT, conf);
		} else {
			set_WineConfig_text( gtk_entry_get_text(GTK_ENTRY(PCMEntry)), ALSA_DEV_TEXT, conf );
			set_WineConfig_text( gtk_entry_get_text(GTK_ENTRY(CTLEntry)), ALSA_CTL_TEXT, conf );
		}
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OSSAdetectButton)) ) {
			set_WineConfig_text("!AUTO", OSS_DEV_TEXT, conf);
			set_WineConfig_text("!AUTO", OSS_MXR_TEXT, conf);
		} else {
			set_WineConfig_text( gtk_entry_get_text(GTK_ENTRY(DSPEntry)), OSS_DEV_TEXT, conf );
			set_WineConfig_text( gtk_entry_get_text(GTK_ENTRY(MixerEntry)), OSS_MXR_TEXT, conf );
		}
		
		conf->vars[FREETYPE_VAR]      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(FreetypeButton));
		conf->vars[SHMWINESERVER_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SHMButton));
		conf->vars[RENICEWINESERVER_VAR] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ReniceButton));
		conf->vars[PTHREADS_VAR] = gtk_combo_box_get_active(GTK_COMBO_BOX(PthreadsBox));
		
		wineconfig_SetGlobalConfig(conf);
		
	} while(response == GTK_RESPONSE_APPLY);
	
	free_WineConfig(conf);
}

void globalconf_snddrv_changed(void)
{
	GtkWidget* ALSAButton = glade_xml_get_widget(globalconfdialog, "ALSAButton");
	GtkWidget* FullDuplexButton = glade_xml_get_widget(globalconfdialog, "FullDuplexButton");
	
	gtk_widget_set_sensitive(
		FullDuplexButton,
		!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAButton)) );
}

void globalconf_ALSAAdetect_toggled(GtkWidget* ALSAAdetectButton)
{
	GtkWidget* PCMEntry = glade_xml_get_widget(globalconfdialog, "PCMEntry");
	GtkWidget* PCMLabel = glade_xml_get_widget(globalconfdialog, "PCMLabel");
	GtkWidget* CTLEntry = glade_xml_get_widget(globalconfdialog, "CTLEntry");
	GtkWidget* CTLLabel = glade_xml_get_widget(globalconfdialog, "CTLLabel");
	
	gboolean adetect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ALSAAdetectButton));
	
	gtk_widget_set_sensitive(PCMEntry, !adetect);
	gtk_widget_set_sensitive(PCMLabel, !adetect);
	gtk_widget_set_sensitive(CTLEntry, !adetect);
	gtk_widget_set_sensitive(CTLLabel, !adetect);
}

void globalconf_OSSAdetect_toggled(GtkWidget* OSSAdetectButton)
{
	GtkWidget* DSPEntry = glade_xml_get_widget(globalconfdialog, "DSPEntry");
	GtkWidget* DSPLabel = glade_xml_get_widget(globalconfdialog, "DSPLabel");
	GtkWidget* MixerEntry = glade_xml_get_widget(globalconfdialog, "MixerEntry");
	GtkWidget* MixerLabel = glade_xml_get_widget(globalconfdialog, "MixerLabel");
	
	gboolean adetect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OSSAdetectButton));
	
	gtk_widget_set_sensitive(DSPEntry, !adetect);
	gtk_widget_set_sensitive(DSPLabel, !adetect);
	gtk_widget_set_sensitive(MixerEntry, !adetect);
	gtk_widget_set_sensitive(MixerLabel, !adetect);
}
