/***************************************************************************
 *            installdialog.c
 *
 *  Sat Jul 10 00:38:47 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <wait.h>

#include <string.h>

#include <glade/glade.h>
#include <gtk/gtk.h>

#include "wineconfig.h"
#include "winverutils.h"
#include "wineexecutils.h"
#include "prefs.h"

extern GladeXML* installdialog;
extern void refreshTreeView();

void install_runInstallDialog(GtkWidget* InstallButton)
{
    GtkWidget* InstallDialog = glade_xml_get_widget(installdialog, "InstallDialog");
	
	GtkWidget* WinverBox = glade_xml_get_widget(installdialog, "WinverBox");		
	GtkWidget* PthreadsBox = glade_xml_get_widget(installdialog, "PthreadsBox");
	GtkWidget* ReniceButton = glade_xml_get_widget(installdialog, "ReniceButton");
	GtkWidget* WineExecBox = glade_xml_get_widget(installdialog, "WineExecBox");
	
	// set WinverBox to default to "Default", PthreadsBox to "Auto
	gtk_combo_box_set_active(GTK_COMBO_BOX(WinverBox), 0);
	gtk_combo_box_set_active(GTK_COMBO_BOX(PthreadsBox), 0);
	gtk_combo_box_set_active(GTK_COMBO_BOX(WineExecBox), 0);
	
	// set initial value for ReniceButton
	WineConfig _conf;
	WineConfig* conf = &_conf;
	init_WineConfig(conf);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ReniceButton), conf->vars[RENICEWINESERVER_VAR]);
	
	//*** run the dialog
	// response will either be GTK_RESPONSE_OK or GTK_RESPONSE_CANCEL
	gint response = gtk_dialog_run(GTK_DIALOG(InstallDialog));
	gtk_widget_hide(InstallDialog);
	
	if(response != GTK_RESPONSE_OK)
		return;
	
	// args is the array that will be passed to execvp
	uint argnum = 0;
	char** args = malloc(4);
	
	//Wine Version
	gint WineExecActive = gtk_combo_box_get_active(GTK_COMBO_BOX(WineExecBox));
    char* wine_exec = malloc(strlen( WineExecText(WineExecActive) ) + 1);
    strcpy(wine_exec, WineExecText(WineExecActive));	
    printf ( "Using %s\n\n", wine_exec);
	
	args[argnum] = malloc( strlen(WineExecText(WineExecActive)) + 1 );
	strcpy(args[argnum], WineExecText(WineExecActive));
	argnum++;
	
	// if "Big Exe" is checked, append -bigexe
	GtkWidget* BigexeCheck = glade_xml_get_widget(installdialog, "BigexeCheck");
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(BigexeCheck))) {
		args = realloc(args, (argnum + 2) * 4);
		args[argnum] = malloc(8);
		strcpy(args[argnum], "-bigexe ");
		argnum++;
    }
          
    // if Winver is not default, append -winver
    gint WinverActive = gtk_combo_box_get_active(GTK_COMBO_BOX(WinverBox));
    if(WinverActive != WINVER_DEFAULT && WinverActive != -1) {
        args = realloc(args, (argnum + 3) * 4);
		
		args[argnum] = malloc(8);
		strcpy(args[argnum], "-winver");
		argnum++;
		
		args[argnum] = malloc(strlen( WinverText(WinverActive) ) + 1);
		strcpy(args[argnum], WinverText(WinverActive));
		argnum++;
    }
	
	// if Pthreads is not set to "auto", specify with -use-pthreads
    gint PthreadsActive = gtk_combo_box_get_active(GTK_COMBO_BOX(PthreadsBox));
	if(PthreadsActive != PTHREADS_AUTO){
		args = realloc(args, (argnum + 3) * 4);
		args[argnum] = malloc(14);
		strcpy(args[argnum], "-use-pthreads");
		argnum++;
		
		switch(PthreadsActive){
    		case PTHREADS_NO:
				args[argnum] = malloc(3);
				strcpy(args[argnum], "no");
				break;
			case PTHREADS_YES:
				args[argnum] = malloc(4);
				strcpy(args[argnum], "yes");
				break;
		}
		argnum++;
	}
		
    // if Debug is checked, append -debug options
    GtkWidget* DebugCheck = glade_xml_get_widget(installdialog, "DebugCheck");
    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DebugCheck)) ){
        GtkWidget* DebugOptions = glade_xml_get_widget(installdialog, "DebugOptions");
		
		args = realloc(args, (argnum + 2));
		args[argnum] = malloc(7);
		strcpy(args[argnum], "-debug");
		argnum++;
		
		char* options = malloc(strlen( gtk_entry_get_text(GTK_ENTRY(DebugOptions)) ) + 1);
		strcpy( options, gtk_entry_get_text(GTK_ENTRY(DebugOptions)) );
		
		while(strchr(options, '+') != NULL){
			gboolean changeback = FALSE;
			char* tmp = strchr(options, '+');
			if(strchr(tmp, ' ')) {
				*(strchr(tmp, ' ')) = '\0';
				changeback = TRUE;
			}
			
			args = realloc(args, (argnum + 2));
			args[argnum] = malloc(strlen(tmp) + 1);
			strcpy(args[argnum], tmp);
			argnum++;
			
			tmp = tmp + strlen(tmp);
			
			if(changeback)
				tmp[0] = ' ';
		}
		
		free(options);
    }
           
    // if specify directory is checked, append -use-dos-cwd
    GtkWidget* SDCheck = glade_xml_get_widget(installdialog, "SpecifydirectoryCheck");
    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SDCheck)) ) {
     	GtkWidget* SDBox = glade_xml_get_widget(installdialog, "SpecifydirectoryBox");
		
		args = realloc(args, (argnum + 3) * 4);
		args[argnum] = malloc(13);
		strcpy(args[argnum], "-use-dos-cwd");
		argnum++;
		
		args[argnum] = malloc(4 + strlen( gtk_entry_get_text(GTK_ENTRY(SDBox)) ) + 1);
		strcpy(args[argnum], gtk_entry_get_text(GTK_ENTRY(SDBox)) );
		argnum++;
    }
	
	// anyone know a better way to do this?  my logic is failing me.
	gboolean eject = TRUE;
	char* ejectpref = prefs_GetPref("Eject");
	if(ejectpref != NULL){
		if(strcmp(ejectpref, "No") == 0)
			eject = FALSE;
		free(ejectpref);
	}
	
	// cdrom ejection monitoring
	if(strcmp(wine_exec, "winelauncher") == 0){
		printf("I found winelauncher");
	}
	if(strcmp(wine_exec, "cedega") == 0){
		if(eject){
			args = realloc(args, (argnum + 2) * 4);
			args[argnum] = malloc(22);
			strcpy(args[argnum], "--monitor-cdrom-eject");
			argnum++;
		}
	}
            
    // append the executable name
    GtkWidget* InstallExecBox = glade_xml_get_widget(installdialog, "InstallExecBox");
	args = realloc(args, (argnum + 3) * 4);
	
	args[argnum] = malloc(3);
	strcpy(args[argnum], "--");
	argnum++;
	
	args[argnum] = malloc(4 + strlen( gtk_entry_get_text(GTK_ENTRY(InstallExecBox)) ) + 1);
	//sprintf( args[argnum], "%s", gtk_entry_get_text(GTK_ENTRY(InstallExecBox)) );
	strcpy(args[argnum], gtk_entry_get_text(GTK_ENTRY(InstallExecBox)) );
    argnum++;
	
	args[argnum] = NULL;
	
    // run the installer
	printf( "Running installer: %s\n\n", gtk_entry_get_text(GTK_ENTRY(InstallExecBox)) );

	GtkWidget* ParentWin = gtk_widget_get_toplevel(InstallButton);
	gtk_window_iconify(GTK_WINDOW(ParentWin));	
	
	// this ingenious bit of code c/o of the GTK+ Reference Manual, a.k.a. my Bible
	while( gtk_events_pending() )
	  gtk_main_iteration ();
	//
	
	pid_t forkid = fork();
	if(forkid == 0){
		execvp(wine_exec, args);
		_exit(EXIT_FAILURE);
	} else if(forkid != -1) {
		
		if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ReniceButton)) ) {
			FILE* ps = popen("ps -C wineserver -o pid --noheaders", "r");
			if(ps != NULL){
				char* cmd = NULL;
				char* pid = malloc(10);
					while(fgets(pid, 10, ps) != NULL){
						*(strrchr(pid, '\n')) = '\0';
						cmd = realloc(cmd, 13 + strlen(pid) + 1);
						sprintf(cmd, "renice 10 -p %s", pid);
						system(cmd);
					}
				free(cmd);
				free(pid);
			pclose(ps);
			} // if ps != NULL
		} // if renice
		
		waitpid(forkid, NULL, FALSE);
		refreshTreeView();
	} else
		printf("Error creating child process for installer\n");
	
	uint i;
	for(i = 0; i <= argnum; i++)
		free(args[i]);
	free(args);
	
	free(wine_exec);
	
	gtk_window_deiconify(GTK_WINDOW(ParentWin));
        
} // run_InstallDialog

void install_onBrowseClicked(void)
{
	// For some reason, libglade (2.4.0) doesn't recognize GtkFileChooserDialog.
	// They're all the same though so there is no need to create one in glade when
	// I can do so right here
	GtkWidget* FileBrowser = gtk_file_chooser_dialog_new ("Select Installer Executable",
				      NULL,
				      GTK_FILE_CHOOSER_ACTION_OPEN,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
				      NULL);
	
	GtkFileFilter* allfiles = gtk_file_filter_new();
	gtk_file_filter_set_name(allfiles, "All Files");
	gtk_file_filter_add_pattern(allfiles, "*");
	
	GtkFileFilter* exes = gtk_file_filter_new();
	gtk_file_filter_set_name(exes, "EXE files");
	gtk_file_filter_add_pattern(exes, "*.exe");
	
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileBrowser), allfiles);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileBrowser), exes);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(FileBrowser), exes);
	
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(FileBrowser), getenv("HOME"));
	
	// if response is "OK" we're gonna copy the filename into InstallExecBox
	gint response = gtk_dialog_run(GTK_DIALOG(FileBrowser));
	if(response == GTK_RESPONSE_OK)
	{
		GtkWidget* InstallExecBox = glade_xml_get_widget(installdialog, "InstallExecBox");
		gchar* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileBrowser));
		if(filename != NULL)
		{
			gtk_entry_set_text(GTK_ENTRY(InstallExecBox), filename);
		}
		g_free(filename);
	}
	gtk_widget_destroy(FileBrowser);

}
	

void install_onCmdlineCheck(GtkWidget* CmdlineCheck)
{
	GtkWidget* CmdlineBox = glade_xml_get_widget(installdialog, "CmdlineBox");
	GtkWidget* OptionsTable = glade_xml_get_widget(installdialog, "OptionsTable");
	GtkWidget* InstallExecBox = glade_xml_get_widget(installdialog, "InstallExecBox");
	GtkWidget* InstallerBrowseButton = glade_xml_get_widget(installdialog, "InstallerBrowseButton");
	
	// if the box is checked we enable CmdlineBox and disable all of the other options,
	// vice-versa if the box is unchecked
	if(CmdlineCheck != NULL && CmdlineBox != NULL){
		
		gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CmdlineCheck));
		gtk_widget_set_sensitive(CmdlineBox, active);
		gtk_widget_set_sensitive(OptionsTable, !active);
		gtk_widget_set_sensitive(InstallExecBox, !active);
		gtk_widget_set_sensitive(InstallerBrowseButton, !active);
	}
}

void install_onDebugCheck(GtkWidget* DebugCheck)
{
	GtkWidget* DebugOptions = glade_xml_get_widget(installdialog, "DebugOptions");
	
	// if the box is checked, enable the DebugOptions field and add a hint (+all)
	// if unchecked, change DebugOptions to "" and disable the field
	if(DebugCheck != NULL && DebugOptions != NULL)
	{
		gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DebugCheck));
		gtk_widget_set_sensitive(DebugOptions, active);
		if(active)
		{
			gtk_entry_set_text(GTK_ENTRY(DebugOptions), "+all");
		} else {
			gtk_entry_set_text(GTK_ENTRY(DebugOptions), "");
		}
	}
}

void install_onSpecifydirectoryCheck(GtkWidget* SdCheck)
{
	GtkWidget* SdBox = glade_xml_get_widget(installdialog, "SpecifydirectoryBox");
	
	// if SpecifydirectoryBox is checked, enable the "specify directory" field
	if(SdCheck != NULL && SdBox != NULL)
	{
		gtk_widget_set_sensitive(SdBox,
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SdCheck)));
	}
}
