/***************************************************************************
 *            main.c
 *
 *  Sun Jul 18 20:34:42 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


/* Template for this file created by Anjuta version 1.2.2 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <wait.h>

#include <sys/types.h>
#include <dirent.h>

#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "desktop_entry.h"
#include "wineconfig.h"
#include "gameconfdialog.h"
#include "gametree.h"
#include "favorites.h"
#include "prefs.h"

GladeXML* mainwindow;
GladeXML* installdialog;
GladeXML* prefsdialog;
GladeXML* gameconfdialog;
GladeXML* globalconfdialog;
GladeXML* shortcuts;

gboolean running = FALSE;

// tab indexes for the notebook
enum {
	FAVORITES_VIEW,
	TREE_VIEW
};

// reads prefs and figures out the path to our shortcuts
char* get_programspath();

void show_Warning(char* message, GtkWidget* parent);

void RemoveFromFavorites(void);
void AddToFavorites(void);

void run_program(void);
void configure_game(void);

void refreshTreeView();
void initTreeView();
void refreshViews(void);

char* get_programspath()
{	
	char kde[]     = ".kde/share/applnk/TransGaming/Programs";
	char kde3[]    = ".kde3/share/applnk/TransGaming/Programs";
	char kde3mdk[] = ".kde3/share/applnk-mdk/TransGaming/Programs";
	char kdemdk[]  = ".kde/share/applnk-mdk/TransGaming/Programs";
	char gnome[]   = ".gnome/apps/TransGaming/Programs";
	
	char* directories[5] = { kde, kde3, kde3mdk, kdemdk, gnome };
	
	char* path = prefs_GetPref("ProgramsDir");
	
	if(path == NULL)
		return NULL;
	
	uint istart;
	uint iend;
	if(strcmp(path, "Auto") == 0){
		istart = 0;
		iend   = 4;
	} else if(strcmp(path, "Debian/Mandrake") == 0){
		istart = 2;
		iend   = 3;
	} else if(strcmp(path, "Gnome") == 0){
		istart = 4;
		iend   = 4;
	} else if(strcmp(path, "KDE") == 0){
		istart = 0;
		iend   = 1;
	} else {
		return path;
	}
		
	uint i;
	for(i = istart; i <= iend; i++){

		path = realloc(path, strlen(getenv("HOME")) + 1 + strlen(directories[i]) + 1);
		sprintf(path, "%s/%s", getenv("HOME"), directories[i]);
		
		if(access(path, F_OK) == 0)
			return path;
	}
	printf("Either could not find specified directory, or autodetection failed.\n");
	free(path);
	
	return NULL;
}

void show_Warning(char* message, GtkWidget* parent)
{
	GtkWidget* Warning = gtk_message_dialog_new(
		GTK_WINDOW(parent),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_OK,
		message);
	
	gtk_dialog_run(GTK_DIALOG(Warning));
	gtk_widget_destroy(Warning);
}

void RemoveFromFavorites(void)
{
	GtkWidget* FavoritesView = glade_xml_get_widget(mainwindow, "FavoritesView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(FavoritesView));
	
	char* entrypath = tree_GetPathFromSelection(selection);
	if(entrypath == NULL)
		return;
	
	char* cmd = malloc(4 + strlen(entrypath) + 1 + 1);
	sprintf(cmd, "rm \"%s\"", entrypath);
	system(cmd);
	
	free(cmd);
	free(entrypath);
	favorites_TreeViewFill(GTK_TREE_VIEW(FavoritesView));
}

void AddToFavorites(void)
{
	GtkWidget* MainNotebook = glade_xml_get_widget(mainwindow, "MainNotebook");
	
	GtkWidget* GameTreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(GameTreeView));
	
	switch(tree_GetTypeFromSelection(selection)) {
		case DIR_TYPE:
		case P2P_DIR_TYPE:
			return;
		
		case P2P_TYPE:
			show_Warning(
				"Sorry, Point2Play games cannot be added to favorites at this time.",
				glade_xml_get_widget(mainwindow, "MainWindow")
			);
			return;
	}
	
	char* entrypath = tree_GetPathFromSelection(selection);
	if(entrypath == NULL)
		return;
	
	char* cmd = malloc(4 + strlen(entrypath) + 3 + strlen(getenv("HOME")) + 22 + 1);
	sprintf(cmd, "cp \"%s\" \"%s/.grapevine/favorites\"", entrypath, getenv("HOME"));
	system(cmd);
	free(cmd);
	
	free(entrypath);
	
	GtkWidget* FavoritesView = glade_xml_get_widget(mainwindow, "FavoritesView");
	favorites_TreeViewFill(GTK_TREE_VIEW(FavoritesView));
	gtk_notebook_set_current_page(GTK_NOTEBOOK(MainNotebook), FAVORITES_VIEW);
}

gboolean renice_wineserver(void)
{
	if(!running)
		return FALSE;
	printf("Attempting to renice_wineserver . . .\n");
	FILE* ps = popen("ps -C wineserver -o pid --noheaders", "r");
	if(ps != NULL){
		char* cmd = NULL;
		char* pid = malloc(10);
		
		if(fgets(pid, 10, ps) == NULL){
			free(pid);
			printf("No wineserver instances found.  Trying again in 5000ms\n");
			return TRUE;
		}
		
		do {
			*(strrchr(pid, '\n')) = '\0';
			cmd = realloc(cmd, 13 + strlen(pid) + 1);
			sprintf(cmd, "renice 10 -p %s", pid);
			system(cmd);
		} while(fgets(pid, 10, ps) != NULL);
		
		free(cmd);
		free(pid);
		pclose(ps);
	} // if ps != NULL
	else{
		printf("Couldn't run \'ps\'.  Aborting\n");
		return FALSE;
	}
	printf("Success!\n");
	return FALSE;
}

void deiconify(void)
{
	GtkWidget* MainWindow = glade_xml_get_widget(mainwindow, "MainWindow");
	gtk_window_deiconify(GTK_WINDOW(MainWindow));
	running = FALSE;
}

// runs the currently selected program
void run_program(void)
{
	// pick a GtkTreeView based on what tab we're looking at
	GtkWidget* MainNotebook = glade_xml_get_widget(mainwindow, "MainNotebook");
	gint index = gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook));
	
	GtkWidget* TreeView;	
	if(index == TREE_VIEW)
		TreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	else
		TreeView = glade_xml_get_widget(mainwindow, "FavoritesView");
	
	// get the selection, get the path for the shortcut, use it to find the Exec line
	GtkTreeSelection* selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(TreeView));		
	
	// get path value
	char* entrypath = tree_GetPathFromSelection(selection);
	if(entrypath == NULL)
		return;
	
	// create args[] array to store all command line parameters
	char** args = malloc(4);
	uint argnum = 0;
	
	// stores whether we renice the wineserver
	gboolean renice = FALSE;
	
	char* p2pshortcut;
	char* cmd = NULL;
	switch(tree_GetTypeFromSelection(selection))
	{
		case DIR_TYPE:
		case P2P_DIR_TYPE:
			free(entrypath);
			free(args);
			
			return;
		// case DIR_TYPE
		
		case P2P_TYPE:
			p2pshortcut = strchr(entrypath, '/');
			*p2pshortcut = '\0';
			p2pshortcut++;
			
			argnum = 4;
			args = realloc(args, (argnum + 1) * 4);
			
			args[0] = malloc(6);
			strcpy(args[0], "Point2Play");
			
			args[1] = malloc(5);
			strcpy(args[1], "-run");
			
			args[2] = malloc(strlen(entrypath) + 1);
			strcpy(args[2], entrypath);
			
			args[3] = malloc(strlen(p2pshortcut) + 1);
			strcpy(args[3], p2pshortcut);
			
			args[4] = NULL;
		
			free(entrypath);		
		
			break;		
		// case P2P_TYPE
		
		case SHORTCUT_TYPE:
			// get the name of the .exe
			cmd = desktop_entry_GetValue("Exec", entrypath);
			free(entrypath);
	
			if(cmd == NULL){
				free(args);
				return;
			}
	
			// make a string out of the name of the exe file
			char* exe = strrchr(cmd, '/');
			
			while( !strstr(exe, ".exe") ) {
				char* tmp = exe;
				*tmp = '\0';
				exe = strrchr(cmd, '/');
				*tmp = '/';
			}
			
			*(strstr(exe, ".exe") + 4) = '\0';
			exe = exe + 1;
			
			// create the config struct
			WineConfig _conf;
			WineConfig* conf = &_conf;
			init_WineConfig(conf);
		
			if(!wineconfig_GetGameConfig(conf, exe)){
				free(cmd);
				free(args);
				return;	
			}
			
			// change cmd back
			exe[strlen(exe)] = '"';
			
			// for now, if no setting for a WineX version is found, use "cedega"
			if(conf->text[WINEEXEC_TEXT] == NULL)
				set_WineConfig_text("cedega", WINEEXEC_TEXT, conf);
			
			// add exec name to args[]
			args[argnum] = malloc( strlen(conf->text[WINEEXEC_TEXT]) + 1);
			strcpy(args[argnum], conf->text[WINEEXEC_TEXT]);
			argnum++;
			
			
			// add -bigexe argument
			if(conf->vars[BIGEXE_VAR]) {
				args = realloc(args, (argnum + 2) * 4);
				args[argnum] = malloc(8);
				strcpy(args[argnum], "-bigexe");
				argnum++;
			}
			
			// add -use-pthreads argument
			if(conf->vars[PTHREADS_VAR] != PTHREADS_AUTO){
				args = realloc(args, (argnum + 3) * 4);
				args[argnum] = malloc(14);
				strcpy(args[argnum], "-use-pthreads");
				argnum++;
				switch(conf->vars[PTHREADS_VAR]){
					case PTHREADS_YES:
						args[argnum] = malloc(4);
						strcpy(args[argnum], "yes");
						argnum++;
						break;
					case PTHREADS_NO:
						args[argnum] = malloc(3);
						strcpy(args[argnum], "no");
						argnum++;
						break;
				}
			}
				
			// if workdir is unspecified, look for a workdir argument in cmd
			if(conf->text[WORKDIR_TEXT] == NULL){
				char str1[] = "-workdir";
				char str2[] = "-use-dos-cwd";
				char* strs[2] = { str1, str2 };
				
				uint i;
				for(i = 0; i < 2; i++){
					if(strstr(cmd, strs[i]) != NULL){
						char* tmp = strchr(strstr(cmd, strs[i]), '"') + 1;
						*(strchr(tmp, '"')) = '\0';
						
						set_WineConfig_text(tmp, WORKDIR_TEXT, conf);
											
						tmp[strlen(tmp)] = '"';
						break;
					}
				}
			}
			
			// append the -use-dos-cwd argument
			if(conf->text[WORKDIR_TEXT] != NULL){
				args = realloc(args, (argnum + 3) * 4);
				
				args[argnum] = malloc(13);
				strcpy(args[argnum], "-use-dos-cwd");
				argnum++;
				
				args[argnum] = malloc(4 +strlen(conf->text[WORKDIR_TEXT]) + 1);
				strcpy(args[argnum], conf->text[WORKDIR_TEXT]);
				argnum++;
			}
			
			renice = conf->vars[RENICEWINESERVER_VAR];
	
			if( strcmp(conf->text[WINEEXEC_TEXT], "cedega") == 0 )
			{
				char* ejectpref = prefs_GetPref("Eject");
				if(ejectpref != NULL){
					if(strcmp(ejectpref, "Yes") == 0){
						args = realloc(args, (argnum + 2) * 4);
						args[argnum] = malloc(22);
						strcpy(args[argnum], "--monitor-cdrom-eject");
						argnum++;
					}
					free(ejectpref);
				}
			}
			
			// trim the final quote and place exepath after the starting quote for the exe
			char* exepath = strrchr(cmd, '"');
			char* cmdextra = NULL;
			if(exepath + 1 != NULL){
				// locate any additional arguments in the shortcut
				cmdextra = exepath + 1;
			}
			exepath[0] = '\0';
			exepath = strrchr(cmd, '"') + 1;
			
			// add command line parameters
			//
			if(conf->text[CMDLINE_TEXT] != NULL)
			{
				char* cmdlinefull = malloc(strlen(cmdextra) + 1 + strlen(conf->text[CMDLINE_TEXT]) + 1);
				sprintf(cmdlinefull, "%s %s", cmdextra, conf->text[CMDLINE_TEXT]);
				set_WineConfig_text(cmdlinefull, CMDLINE_TEXT, conf);
			}
			else if(conf->text[CMDLINE_TEXT] == NULL && cmdextra != NULL)
				set_WineConfig_text(cmdextra, CMDLINE_TEXT, conf);
			
						
			// add the parameter for the exe, then add a NULL pointer
			args = realloc(args, (argnum + 3) * 4);
			args[argnum] = malloc(strlen(exepath) + 1);
			strcpy(args[argnum], exepath);
			argnum++;
			
			if(conf->text[CMDLINE_TEXT] != NULL)
			{
				printf("%s\n", conf->text[CMDLINE_TEXT]);
				args = realloc(args, (argnum + 2) * 4);
				args[argnum] = malloc(strlen(conf->text[CMDLINE_TEXT]) + 1);
				strcpy(args[argnum], conf->text[CMDLINE_TEXT]);
				argnum++;
			}
			
			args[argnum] = NULL;
			
			free(cmd);
			free_WineConfig(conf);
			
			break;
			// case SHORTCUT_TYPE
		} // switch type
	
	char* path = tree_GetPathFromSelection(selection);
	printf("Launching %s: %s\n\n", args[0], path);
	free(path);
	
	// iconify the window
	GtkWidget* MainWindow = glade_xml_get_widget(mainwindow, "MainWindow");
	gtk_window_iconify(GTK_WINDOW(MainWindow));
	
	// from the GTK+ Reference Manual
	while( gtk_events_pending() )
		gtk_main_iteration();
	
	// this runs the actual command.  use fork/exec instead of system() so we can renice the wineserver
	pid_t forkid = fork();
	if(forkid == 0){
		execvp(args[0], args);
		_exit(EXIT_FAILURE);	
	} else if(forkid != -1) {
		running = TRUE;
		// renice the wineserver
		if(renice){
			gtk_timeout_add(5000, (GtkFunction)renice_wineserver, NULL);
		} // if renice
		
	} else
		printf("Error creating child process for Cedega\n");
	
	// free all of the args[] strings
	uint i;
	for(i = 0; i <= argnum; i++)
		free(args[i]);
	free(args);
	
	// deiconify when the game is closed
	g_child_watch_add(forkid, (GChildWatchFunc)deiconify, GTK_WINDOW(MainWindow));
	
} // run_program

// configures the currently selected program
void configure_game(void)
{
	GtkWidget* MainNotebook = glade_xml_get_widget(mainwindow, "MainNotebook");
	gint index = gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook));
	
	GtkWidget* TreeView;
	if(index == TREE_VIEW){
		TreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	} else {
		TreeView = glade_xml_get_widget(mainwindow, "FavoritesView");
	}
		
	GtkTreeSelection* selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(TreeView));
	
	switch(tree_GetTypeFromSelection(selection)) {	
		case DIR_TYPE:
		case P2P_DIR_TYPE:
			return;
		
		case P2P_TYPE:
			show_Warning(
				"Sorry, configuration of Point2Play games is not supported yet.  Please run Point2Play for configuration.",
				glade_xml_get_widget(mainwindow, "MainWindow")
			);
			return;
	}
	
	char* entrypath = tree_GetPathFromSelection(selection);
	if(entrypath == NULL)
		return;
	
	char* cmd = desktop_entry_GetValue("Exec", entrypath);
	free(entrypath);
	
	char* exe = strrchr(cmd, '/');
	*(strstr(exe, ".exe") + 4) = '\0';
	exe = exe + 1;
	
	gameconf_runGameConfDialog(exe);
		
	free(cmd);
}

// Shows the about box
void show_about (void)
{
	GtkWidget* AboutBox = glade_xml_get_widget(mainwindow, "AboutBox");
	if(AboutBox != NULL){
		gtk_dialog_run(GTK_DIALOG(AboutBox));
		gtk_widget_hide(AboutBox);
	}
}

// abstracted refilling of treestore
void refreshTreeView()
{
    GtkWidget* TreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	char* programspath = get_programspath();
    tree_TreeViewFill(GTK_TREE_VIEW(TreeView), programspath );
}

// for the refresh button
void refreshViews(void)
{
	GtkWidget* FavoritesView = glade_xml_get_widget(mainwindow, "FavoritesView");
	
	refreshTreeView();
	favorites_TreeViewFill(GTK_TREE_VIEW(FavoritesView));
}
	

// prepare and fill the treeview
void initTreeViews()
{
    GtkWidget* TreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkWidget* FavoritesView = glade_xml_get_widget(mainwindow, "FavoritesView");
	
	GtkWidget* views[2] = {TreeView, FavoritesView};
	uint i;
	
	for(i = 0; i <= 1; i++)
	{	
		GtkCellRenderer* name_renderer = gtk_cell_renderer_text_new ();
		GtkCellRenderer* icon_renderer = gtk_cell_renderer_pixbuf_new();
	
		GtkTreeViewColumn* column = gtk_tree_view_column_new();
	
		// pack our renderers into the column
		gtk_tree_view_column_pack_start(column, icon_renderer, FALSE);
		gtk_tree_view_column_pack_end(column, name_renderer, FALSE);
				
		// set their attributes
		gtk_tree_view_column_add_attribute(column, icon_renderer, "pixbuf", ICON_COLUMN);
		gtk_tree_view_column_add_attribute(column, name_renderer, "text", NAME_COLUMN);
		
		// this makes it look much nicer
		gtk_tree_view_column_set_spacing(column, 5);
	
		// add the column to the tree view
		gtk_tree_view_append_column (GTK_TREE_VIEW(views[i]), column);
	}
	
	// fill the tree view
	char* programspath = get_programspath();
	printf("Using %s as programs directory\n\n", programspath);
	tree_TreeViewFill(GTK_TREE_VIEW(TreeView), programspath );
	
	// fill the favorites view
	favorites_TreeViewFill(GTK_TREE_VIEW(FavoritesView));
}

// callback for switching tabs
void MainNotebook_switch_page(GtkWidget* MainNotebook)
{
	GtkWidget* AddToFavoritesButton = glade_xml_get_widget(mainwindow, "AddToFavoritesButton");
	GtkWidget* RemoveFromFavoritesButton = glade_xml_get_widget(mainwindow, "RemoveFromFavoritesButton");
	GtkWidget* NewShortcutButton = glade_xml_get_widget(mainwindow, "NewShortcutButton");
	GtkWidget* NewGroupButton = glade_xml_get_widget(mainwindow, "NewGroupButton");
	GtkWidget* DeleteShortcutButton = glade_xml_get_widget(mainwindow, "DeleteShortcutButton");
	
	gint index = gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook));
	
	gtk_widget_set_sensitive(AddToFavoritesButton, index != TREE_VIEW);
	gtk_widget_set_sensitive(RemoveFromFavoritesButton, index == TREE_VIEW);
	gtk_widget_set_sensitive(NewShortcutButton, index != TREE_VIEW);
	gtk_widget_set_sensitive(NewGroupButton, index != TREE_VIEW);
	gtk_widget_set_sensitive(DeleteShortcutButton, index != TREE_VIEW);
}

// obvious
gboolean on_MainWindow_destroy_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  return FALSE;
}

int main (int argc, char *argv[])
{
	/*** needs to be enabled for gettext support
	#ifdef ENABLE_NLS
		bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
		textdomain (PACKAGE);
	#endif
	***/
	
	gtk_init(&argc, &argv);
	
	// if the prefs file doesn't exist, make it
	char* path = malloc(strlen(getenv("HOME")) + 21 + 1);
	sprintf(path, "%s/.grapevine/prefs", getenv("HOME"));
	
	if(access(path, F_OK) != 0)
		prefs_MakePrefsFile();
	
	// if the favorites directory doesn't exist, make it
	sprintf(path, "%s/.grapevine/favorites", getenv("HOME"));
	if(access(path, F_OK) != 0){
		char* cmd = malloc(6 + strlen(path) + 1);
		sprintf(cmd, "mkdir %s", path);
		
		system(cmd);
		free(cmd);
	}
	
	free(path);
	
	// init glade
	glade_init ();
	
	// load each of the dialogs into a GladeXML object
	mainwindow = glade_xml_new (PACKAGE_DATA_DIR"/grapevine/glade/grapevine.glade", NULL, NULL);
	installdialog = glade_xml_new(PACKAGE_DATA_DIR"/grapevine/glade/installdialog.glade", NULL, NULL);
	prefsdialog = glade_xml_new(PACKAGE_DATA_DIR"/grapevine/glade/prefsdialog.glade", NULL, NULL);
	gameconfdialog = glade_xml_new(PACKAGE_DATA_DIR"/grapevine/glade/gameconfdialog.glade", NULL, NULL);
	globalconfdialog = glade_xml_new(PACKAGE_DATA_DIR"/grapevine/glade/globalconfdialog.glade", NULL, NULL);
	shortcuts = glade_xml_new(PACKAGE_DATA_DIR"/grapevine/glade/shortcuts.glade", NULL, NULL);

	// connect all signals
	glade_xml_signal_autoconnect(mainwindow);
	glade_xml_signal_autoconnect(installdialog);
	glade_xml_signal_autoconnect(prefsdialog);
	glade_xml_signal_autoconnect(gameconfdialog);
	glade_xml_signal_autoconnect(globalconfdialog);
	glade_xml_signal_autoconnect(shortcuts);
    
	// load the treeview
    initTreeViews();
	
	// glade is stupid
	GtkWidget* tgimage = gtk_image_new_from_file(PACKAGE_PIXMAPS_DIR"/transgaming.png");
	GtkWidget* GlobalConfigButton = glade_xml_get_widget(mainwindow, "GlobalConfigButton");
	
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(GlobalConfigButton), tgimage);
	gtk_widget_show(tgimage);
	
	// version number for the about box
	GtkWidget* VersionLabel = glade_xml_get_widget(mainwindow, "VersionLabel");
	gtk_label_set_markup(GTK_LABEL(VersionLabel), "\n<span size=\"20000\"><b>Grapevine "VERSION"</b></span>\n");
	
	// enter the main loop
	gtk_main ();
	
	return 0;
}
