/***************************************************************************
 *            prefs.c
 *
 *  Mon Jul 19 20:42:40 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "prefs.h"

extern GladeXML *prefsdialog;
extern void refreshTreeView();

// the program groups location combo box in the prefs dialog
enum {
	PROGDIR_AUTO,
	PROGDIR_MDK,
	PROGDIR_GNOME,
	PROGDIR_KDE,
	PROGDIR_CUSTOM
};

// the setting for cd eject monitoring
enum {
	EJECT_DEFAULT,
	EJECT_ON,
	EJECT_OFF
};

void prefs_ProgPathBrowse_clicked(GtkWidget* BrowseButton)
{
	// until libglade supports GtkFileBrowserDialog . . .
	GtkWidget* FileBrowser = gtk_file_chooser_dialog_new(
		"Select Program Groups directory",
		NULL,
		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
		NULL);
	
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(FileBrowser), getenv("HOME"));
	
	gint response = gtk_dialog_run(GTK_DIALOG(FileBrowser));
	if(response == GTK_RESPONSE_OK)
	{
		GtkWidget* CustomProgDirBox = glade_xml_get_widget(prefsdialog, "CustomProgDirBox");
		gchar* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileBrowser));
		if(filename != NULL)
			gtk_entry_set_text(GTK_ENTRY(CustomProgDirBox), filename);
		g_free(filename);
	}
	gtk_widget_destroy(FileBrowser);
}

void prefs_ProgDirBox_changed(GtkWidget* ProgDirBox)
{
	GtkWidget* CustomProgDirBox = glade_xml_get_widget(prefsdialog, "CustomProgDirBox");
	GtkWidget* CustomProgDirBrowse = glade_xml_get_widget(prefsdialog, "CustomProgDirBrowse");
	
	gboolean custom = (gtk_combo_box_get_active(GTK_COMBO_BOX(ProgDirBox)) == PROGDIR_CUSTOM);
	gtk_widget_set_sensitive(CustomProgDirBox, custom);
	gtk_widget_set_sensitive(CustomProgDirBrowse, custom);
	
	if(!custom)
		gtk_entry_set_text(GTK_ENTRY(CustomProgDirBox), "");
}
		

void prefs_runPrefsDialog(void)
{
	GtkWidget* ProgDirBox = glade_xml_get_widget(prefsdialog, "ProgDirBox");
	GtkWidget* EjectBox = glade_xml_get_widget(prefsdialog, "EjectBox");
	GtkWidget* P2PButton = glade_xml_get_widget(prefsdialog, "P2PButton");
	
	// set initial value for including P2P games
	uint prevp2p = 1;
	char* p2ppref = prefs_GetPref("IncludeP2PGames");
	if(p2ppref != NULL)
		if(*p2ppref == 'N')
			prevp2p = 0;
	free(p2ppref);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(P2PButton), prevp2p);
	
	// set initial value for programs directory box
	int prevprogdir;
	char* programsdir = prefs_GetPref("ProgramsDir");
	if(programsdir != NULL){
		if(strcmp(programsdir, "Auto") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_AUTO);
		else if(strcmp(programsdir, "Debian/Mandrake") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_MDK);
		else if(strcmp(programsdir, "Gnome") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_GNOME);
		else if(strcmp(programsdir, "KDE") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_KDE);
		else
			gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_CUSTOM);
		prevprogdir = gtk_combo_box_get_active(GTK_COMBO_BOX(ProgDirBox));
		free(programsdir);
	} else{
		gtk_combo_box_set_active(GTK_COMBO_BOX(ProgDirBox), PROGDIR_AUTO);
		prevprogdir = -1;
	}
	
	// set initial value for eject monitoring box
	int preveject;
	char* eject = prefs_GetPref("Eject");
	if(eject != NULL){
		if(strcmp(eject, "Default") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(EjectBox), EJECT_DEFAULT);
		else if(strcmp(eject, "On") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(EjectBox), EJECT_ON);
		else if(strcmp(eject, "Off") == 0)
			gtk_combo_box_set_active(GTK_COMBO_BOX(EjectBox), EJECT_OFF);
		preveject = gtk_combo_box_get_active(GTK_COMBO_BOX(EjectBox));
		free(eject);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(EjectBox), EJECT_DEFAULT);
		preveject = -1;
	}
	
	//*** run the dialog
	//
	gint response;
	do {  // while response == GTK_RESPONSE_APPLY
		GtkWidget* PrefsDialog = glade_xml_get_widget(prefsdialog, "PrefsDialog");
		response = gtk_dialog_run(GTK_DIALOG(PrefsDialog));
		gtk_widget_hide(PrefsDialog);
		
		if(response == GTK_RESPONSE_CANCEL)
			return;
		
		// set the value for including P2P games
		gboolean p2p = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(P2PButton));
		if(p2p != prevp2p)
		{
			switch(p2p)
			{
				case TRUE:
					prefs_SetPref("Y", "IncludeP2PGames");
					break;
				case FALSE:
					prefs_SetPref("N", "IncludeP2PGames");
					break;
			}
		}
		
		// set the value for programs directory
		uint progdirindex = gtk_combo_box_get_active(GTK_COMBO_BOX(ProgDirBox));
		if(progdirindex != prevprogdir) {
			
			// I don't know why but the compiler won't let me put this in the conditional.  Just leave it here
			GtkWidget* CustomProgDirBox;
			switch(progdirindex){
				case PROGDIR_CUSTOM:
					CustomProgDirBox = glade_xml_get_widget(prefsdialog, "CustomProgDirBox");
					char* dir = (char*)gtk_entry_get_text(GTK_ENTRY(CustomProgDirBox));
					prefs_SetPref(dir, "ProgramsDir");
					free(dir);
					break;
				case PROGDIR_AUTO:
					prefs_SetPref("Auto", "ProgramsDir");
					break;
				case PROGDIR_GNOME:
					prefs_SetPref("Gnome", "ProgramsDir");
					break;
				case PROGDIR_KDE:
					prefs_SetPref("KDE", "ProgramsDir");
					break;
				case PROGDIR_MDK:
					prefs_SetPref("Debian/Mandrake", "ProgramsDir");
					break;
			} // switch
			refreshTreeView();
		}
	
		// set the value for eject monitoring
		uint ejectindex = gtk_combo_box_get_active(GTK_COMBO_BOX(EjectBox));
		if(ejectindex != preveject) {
			switch(ejectindex){
				case EJECT_DEFAULT:
					prefs_SetPref("Default", "Eject");
					break;
				case EJECT_ON:
					prefs_SetPref("On", "Eject");
					break;
				case EJECT_OFF:
					prefs_SetPref("Off", "Eject");
					break;
			} // switch
		}
		
		refreshTreeView();
	} while (response == GTK_RESPONSE_APPLY);
}
	
	
void prefs_MakePrefsFile()
{
	char path[ strlen(getenv("HOME")) + 18 ];
	sprintf(path, "%s/.grapevine", getenv("HOME"));
		
	// if the directory doesn't exist, create it
	if(access(path, F_OK) != 0){
		if(mkdir(path, S_IRWXU | S_IRGRP | S_IWGRP) != 0){
			printf("Could not create directory: %s\n", path);
			return;
		}
	}
	
	// open or create the prefs file
	strcat(path, "/prefs");
	FILE* prefs = fopen(path, "w");
	if(prefs == NULL){
		printf("Could not create file: %s\n", path);
		return;
	}
	
	// copy initial data into it
	fputs("# Grapevine config file\n\n", prefs);
	fputs("# Whether to include Point2Play games in the tree", prefs);
	fputs("IncludeP2PGames=Y", prefs);
	fputs("# What directory to look for program group entries in\n", prefs);
	fputs("ProgramsDir=Auto\n", prefs);
	fputs("# CD Ejection Monitoring: Default is for installers only\n", prefs);
	fputs("Eject=Default\n", prefs);
	fclose(prefs);
}

char* prefs_GetPref(const char* pref)
{
	char path[strlen(getenv("HOME")) + 18];
	sprintf(path, "%s/.grapevine/prefs", getenv("HOME"));
	
	// open the file
	FILE* prefs = fopen(path, "r");
	if(prefs == NULL){
		printf("Error opening preferences file: %s\n", path);
		return NULL;
	}
	
	char* line = malloc(500);
	
	// look for a match, erase the \n, create a pointer to the byte after the "=" and return
	while(!feof(prefs)){
		fgets(line, 500, prefs);
		if( strncmp(line, pref, strlen(pref)) == 0  &&
			line[strlen(pref)] == '=')
		{
			line[strlen(line) - 1] = '\0';
			char* tmp = line + strlen(pref) + 1;
			
			char* response = malloc(strlen(tmp) + 1);
			strcpy(response, tmp);
			
			fclose(prefs);
			free(line);
			return response;
		}
	}
	// if we find nothing return NULL
	printf("Could not find in %s: %s\n", path, pref);
	fclose(prefs);
	free(line);
	return NULL;
}

void prefs_SetPref(const char* val, const char* pref)
{		
	FILE* prefs;
	char path[strlen(getenv("HOME")) + 18];
	sprintf(path, "%s/.grapevine/prefs", getenv("HOME"));
	
	char* existing = prefs_GetPref(pref);
	// if the value doesn't exist, append it to the file and leave
	if(existing == NULL){
		prefs = fopen(path, "a");
		fprintf(prefs, "%s=%s\n", pref, val);
		fclose(prefs);
		return;
	}
	
	// if the requested change is the same as the existing, do nothing.
	if( strcmp(val, existing) == 0 ){
		free(existing);
		return;
	}
	free(existing);
	
	// we know path exists from GetPref so we don't double-check
	prefs = fopen(path, "r+");
	
	char* line = malloc(500);
	// this is so we don't call strlen() on a null pointer below
	char* startbuf = malloc(1);
	char* endbuf = malloc(1);
	startbuf[0] = '\0';
	endbuf[0] = '\0';
	
	// feof is not included in the condition because we know the value exists
	while(fgets(line, 500, prefs) != NULL ) {
		if( !(
		 	strncmp( line, pref, strlen(pref) ) == 0 
		 	&& line[strlen(pref)] == '='
			)
		){	
			startbuf = realloc(startbuf, strlen(startbuf) + strlen(line) + 1);
			strcat(startbuf, line);
		} else
			break;
	}
	
	// note that we have conveniently skipped over the existing line
	while(fgets(line, 500, prefs) != NULL)
	{
		endbuf = realloc(endbuf, strlen(endbuf) + strlen(line) + 1);
		strcat(endbuf, line);
	}
	free(line);
	
	// rewrite the file with the new line between startbuf and endbuf
	freopen(path, "w", prefs);
	
	fwrite(startbuf, 1, strlen(startbuf), prefs);
	fprintf(prefs, "%s=%s\n", pref, val);
	fwrite(endbuf, 1, strlen(endbuf), prefs);
	
	free(startbuf);
	free(endbuf);
	fclose(prefs);
}
