/***************************************************************************
 *            shortcuts.c
 *
 *  Fri Dec 24 21:03:52 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "gametree.h"
#include "shortcuts.h"
#include "desktop_entry.h"
#include "wineexecutils.h"

extern GladeXML* mainwindow;
extern GladeXML* shortcuts;

extern char* get_programspath();
extern void refreshViews();
extern void refreshTreeView();
extern void show_Warning(char* message, GtkWidget* parent);
 
void shortcuts_AddNewShortcut(void)
{
	GtkWidget* GameTreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(GameTreeView));
	
	// get the program path of the currently selected object, or the base program path if none
	char* programspath = tree_GetPathFromSelection(selection);
	if(programspath == NULL){
		programspath = get_programspath();
		if(programspath == NULL)
			return;
	}
	
	switch(tree_GetTypeFromSelection(selection))
	{
		case SHORTCUT_TYPE:
			*(strrchr(programspath, '/')) = '\0';
			break;
		case P2P_TYPE:
		case P2P_DIR_TYPE:
			free(programspath);
			return;
	}
	
	GtkWidget* NewShortcutDialog = glade_xml_get_widget(shortcuts, "NewShortcutDialog");
	GtkWidget* NameEntry = glade_xml_get_widget(shortcuts, "NewShortcutNameEntry");
	GtkWidget* ExecEntry = glade_xml_get_widget(shortcuts, "NewShortcutExecEntry");
	GtkWidget* GroupHbox = glade_xml_get_widget(shortcuts, "NewShortcutGroupHbox");
	GtkWidget* WineexecBox = glade_xml_get_widget(shortcuts, "NewShortcutWineexecBox");
	
	// Now let's create the group combo box and load the groups
	//
	GtkWidget* GroupBox = gtk_combo_box_new_text();
	gtk_container_add(GTK_CONTAINER(GroupHbox), GroupBox);
	gtk_widget_show(GroupBox);
	
	char* group = strstr(programspath, "Programs") + 8;
	
	// If Programs is followed by a '/'
	if(*group != '\0')
	{
		// Add this group
		gtk_combo_box_append_text(GTK_COMBO_BOX(GroupBox), group);
		char* tmp = strrchr(group, '/');
		
		// Keep truncating at the last '/'
		while(tmp != group){
			*tmp = '\0';
			gtk_combo_box_append_text(GTK_COMBO_BOX(GroupBox), group);
			char* oldtmp = tmp;
			tmp = strrchr(group, '/');
			*oldtmp = '/';
		}
	}
	
	// Add a final '/'
	gtk_combo_box_append_text(GTK_COMBO_BOX(GroupBox), "/");
	gtk_combo_box_set_active(GTK_COMBO_BOX(GroupBox), 0);
	
	// set up the wineexec box
	gtk_combo_box_set_active(GTK_COMBO_BOX(WineexecBox), 0);
	
	gboolean NoName;
	gboolean NoExec;
	gboolean NoPath;
	do {
		// run the dialog
		gint response = gtk_dialog_run(GTK_DIALOG(NewShortcutDialog));
		gtk_widget_hide(NewShortcutDialog);
		if(response != GTK_RESPONSE_OK)
			return;
		
		NoName = !strcmp(gtk_entry_get_text(GTK_ENTRY(NameEntry)), "");
		NoExec = !strcmp(gtk_entry_get_text(GTK_ENTRY(ExecEntry)), "");
		NoPath = (gboolean)(strrchr(gtk_entry_get_text(GTK_ENTRY(ExecEntry)), '/') == NULL);
		
		if(NoPath)
			show_Warning("EXE path is invalid.  Please enter the path from the root directory.", NewShortcutDialog);
		if(NoName)
			show_Warning("You must supply a name for the shortcut.", NewShortcutDialog);
		if(NoExec)
			show_Warning("You must supply an executable for this shortcut.", NewShortcutDialog);
			
	}while(NoName || NoExec || NoPath);
	
	// Create a path for the shortcut
	//
	
	// Get the group text
	gint groupindex = gtk_combo_box_get_active(GTK_COMBO_BOX(GroupBox));
	uint i;
	for(i = 0; i < groupindex; i++) {
		char* tmp = strrchr(group, '/');
		*tmp = '\0';
	}
		
	// Get the shortcut name
	const char* name = gtk_entry_get_text(GTK_ENTRY(NameEntry));
	
	// create the shortcut path and open the file
	char* filename = malloc(strlen(programspath) + 1 + strlen(name) + 8 + 1);
	sprintf(filename, "%s/%s.desktop", programspath, name);
	free(programspath);
	
	FILE* shortcut = fopen(filename, "w");
	if(shortcut == NULL){
		printf("Could not create file: %s\n", filename);
		return;
	}
	free(filename);
	
	// get the other widgets and copy the exec string
	GtkWidget* IconEntry = glade_xml_get_widget(shortcuts, "NewShortcutIconEntry");
	GtkWidget* CustomDirButton = glade_xml_get_widget(shortcuts, "NewShortcutCustomDirButton");
	
	char* exe = malloc(strlen( gtk_entry_get_text(GTK_ENTRY(ExecEntry)) ) + 1);
	strcpy( exe, gtk_entry_get_text(GTK_ENTRY(ExecEntry)) );
	
	// glean the path for the game from the custom entry or from the exe file's directory
	char* path;
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CustomDirButton)) ) {
		// copy the CustomDirEntry text into path
		GtkWidget* CustomDirEntry = glade_xml_get_widget(shortcuts, "NewShortcutCustomDirEntry");
		path = malloc(strlen( gtk_entry_get_text(GTK_ENTRY(CustomDirEntry)) ) + 1);
		strcpy( path, gtk_entry_get_text(GTK_ENTRY(CustomDirEntry)) );
	} else {
		// copy everything before the /.exe file into path
		*strrchr(exe, '/') = '\0';
		path = malloc(strlen(exe) + 1);
		strcpy(path, exe);
		exe[strlen(exe)]  = '/';
	}
		
	// write the new file
	fputs("[Desktop Entry]\n", shortcut);
	fprintf(shortcut, "Name=%s\n", name);
	fprintf(shortcut, "Exec=%s -workdir \"%s\" \"%s\"\n",
		WineExecText( gtk_combo_box_get_active(GTK_COMBO_BOX(WineexecBox)) ),
		path,
		exe
	);
	fputs("Type=Application\n", shortcut);
	fprintf(shortcut, "Path=\"%s\"\n", path);	
	fprintf( shortcut, "Icon=%s\n", gtk_entry_get_text(GTK_ENTRY(IconEntry)) );
	
	fclose(shortcut);
	free(path);
	free(exe);
	
	refreshTreeView();
}


void shortcuts_EditShortcut(void)
{
	// get the currently selected item
	GtkWidget* TreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeView));
	
	// return if selection is not a shortcut
	uint type = tree_GetTypeFromSelection(selection);
	if( type != SHORTCUT_TYPE)
		return;
	
	// load dialog widgets
	GtkWidget* Dialog = glade_xml_get_widget(shortcuts, "EditShortcutDialog");
	GtkWidget* NameEntry = glade_xml_get_widget(shortcuts, "EditShortcutNameEntry");
	GtkWidget* ExecEntry = glade_xml_get_widget(shortcuts, "EditShortcutExecEntry");
	GtkWidget* WineexecBox = glade_xml_get_widget(shortcuts, "EditShortcutWineexecBox");
	GtkWidget* CustomDirButton = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirButton");
	GtkWidget* CustomDirEntry = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirEntry");
	GtkWidget* CustomDirBrowseButton = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirBrowseButton");
	GtkWidget* IconEntry = glade_xml_get_widget(shortcuts, "EditShortcutIconEntry");
	
	// get the shortcut's path
	char* path = tree_GetPathFromSelection(selection);
	if(path == NULL)
		return;
	
	// set initial dialog entry values //
	
	// shorcut name
	char* name = desktop_entry_GetValue("Name", path);
	gtk_entry_set_text(GTK_ENTRY(NameEntry), name);
	free(name);
	
	// exe file
	//
	char* exec = desktop_entry_GetValue("Exec", path);
	
	// truncate after the .exe and put the rest into cmdlineadd
	char* tmp = strstr(exec, ".exe");
	*(tmp + 4) = '\0';
	char* cmdlineadd = tmp + 5;
	
	
	tmp = strrchr(exec, '"');
	tmp++;
	gtk_entry_set_text(GTK_ENTRY(ExecEntry), tmp);
	
	if(strncmp(exec, "cedega ", 7) == 0)
		gtk_combo_box_set_active(GTK_COMBO_BOX(WineexecBox), 0);
	else if(strncmp(exec, "winex3 ", 7) == 0)
		gtk_combo_box_set_active(GTK_COMBO_BOX(WineexecBox), 1);
	else if(strncmp(exec, "winelauncher ", 13) == 0)
		gtk_combo_box_set_active(GTK_COMBO_BOX(WineexecBox), 2);
	else	
		gtk_combo_box_set_active(GTK_COMBO_BOX(WineexecBox), 0);
	free(exec);
	
	// custom working directory
	char* customdir = desktop_entry_GetValue("Path", path);
	if(customdir != NULL)
		if(strcmp(customdir, "") != 0){
			gtk_widget_set_sensitive(CustomDirBrowseButton, TRUE);
			gtk_widget_set_sensitive(CustomDirEntry, TRUE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CustomDirButton), TRUE);
		}
	gtk_entry_set_text(GTK_ENTRY(CustomDirEntry), customdir);
	free(customdir);
	
	// icon path
	char* icon = desktop_entry_GetValue("Icon", path);
	gtk_entry_set_text(GTK_ENTRY(IconEntry), icon);
	free(icon);
	
	gboolean NoName;
	gboolean NoExec;
	gboolean NoPath;
	do {
		// run the dialog
		gint response = gtk_dialog_run(GTK_DIALOG(Dialog));
		gtk_widget_hide(Dialog);
		if(response != GTK_RESPONSE_OK)
			return;
		
		NoName = !strcmp(gtk_entry_get_text(GTK_ENTRY(NameEntry)), "");
		NoExec = !strcmp(gtk_entry_get_text(GTK_ENTRY(ExecEntry)), "");
		NoPath = (gboolean)(strchr(gtk_entry_get_text(GTK_ENTRY(ExecEntry)), '/') == NULL);
		
		if(NoPath)
			show_Warning("EXE path is invalid.  Please enter the path from the root directory.", Dialog);
		if(NoName)
			show_Warning("You must supply a name for the shortcut.", Dialog);
		if(NoExec)
			show_Warning("You must supply an executable for this shortcut.", Dialog);
			
	} while(NoName || NoExec || NoPath);
	
	// set new Name value
	desktop_entry_SetValue(gtk_entry_get_text(GTK_ENTRY(NameEntry)), "Name", path);
	
	// all this is for the new exec value
	//
	exec = malloc(
		strlen(WineExecText( gtk_combo_box_get_active(GTK_COMBO_BOX(WineexecBox)) )) + 10 + 
		strlen( gtk_entry_get_text(GTK_ENTRY(CustomDirEntry)) ) + 5 + 
		strlen( gtk_entry_get_text(GTK_ENTRY(ExecEntry)) ) + 1  +
		strlen(cmdlineadd)
	);
	
	
	
	sprintf(exec, "%s -workdir %s -- \"%s\"%s",	
		WineExecText( gtk_combo_box_get_active(GTK_COMBO_BOX(WineexecBox)) ),
		gtk_entry_get_text(GTK_ENTRY(CustomDirEntry)),
		gtk_entry_get_text(GTK_ENTRY(ExecEntry)),
		cmdlineadd);
	
	desktop_entry_SetValue(exec, "Exec", path);
	free(exec);
	
	// custom path
	desktop_entry_SetValue(gtk_entry_get_text(GTK_ENTRY(CustomDirEntry)), "Path", path);
	
	// icon
	desktop_entry_SetValue(gtk_entry_get_text(GTK_ENTRY(IconEntry)), "Icon", path);
	
}
	

void shortcuts_AddNewGroup(void)
{
	GtkWidget* GameTreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(GameTreeView));
	
	char* path = tree_GetPathFromSelection(selection);
	if(path == NULL){
		path = get_programspath();
		if(path == NULL)
			return;
	}
	
	switch(tree_GetTypeFromSelection(selection))
	{
		case SHORTCUT_TYPE:
			*(strrchr(path, '/')) = '\0';
			break;
		case P2P_TYPE:
		case P2P_DIR_TYPE:
			free(path);
			return;
	}
	
	GtkListStore* grouplist = gtk_list_store_new(1, G_TYPE_STRING);
	GtkTreeIter iter;
	
	char* group = strstr(path, "Programs") + 8;
	if(group[0] == '/')
		group = group + 1;
	
	gtk_list_store_append(grouplist, &iter);
	gtk_list_store_set(grouplist, &iter, 0, group, -1);
	
	
	while(strrchr(group, '/') != NULL){
		*(strrchr(group, '/')) = '\0';
		gtk_list_store_append(grouplist, &iter);
		gtk_list_store_set(grouplist, &iter, 0, group, -1);
	}
	free(path);
	
	GtkWidget* Entry = gtk_combo_box_entry_new();
	gtk_combo_box_set_model(GTK_COMBO_BOX(Entry), GTK_TREE_MODEL(grouplist));
	gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(Entry), 0);
	gtk_combo_box_set_active(GTK_COMBO_BOX(Entry), 0);
	
	GtkWidget* HBox = glade_xml_get_widget(shortcuts, "NewGroupHBox");
	gtk_container_add(GTK_CONTAINER(HBox), Entry);
	gtk_widget_show(Entry);
	
	GtkWidget* Dialog = glade_xml_get_widget(shortcuts, "NewGroupDialog");
	gint response = gtk_dialog_run(GTK_DIALOG(Dialog));
	gtk_widget_hide(Dialog);
	if(response == GTK_RESPONSE_OK)
	{
		
		
		group = malloc(strlen( gtk_entry_get_text(GTK_ENTRY(GTK_BIN(Entry)->child)) ) + 1);
		strcpy( group, gtk_entry_get_text(GTK_ENTRY(GTK_BIN(Entry)->child)) );
		
		path = get_programspath();
		char* tmp = group;
		char* tmp2;
		while(strchr(tmp, '/') != NULL){
			tmp2 = strchr(tmp, '/');
			tmp2[0] = '\0';
			path = realloc(path, strlen(path) + 1 + strlen(tmp) + 1);
			sprintf(path, "%s/%s", path, tmp);
			if(access(path, F_OK) != 0){
				char* cmd = malloc(8 + strlen(path) + 1);
				sprintf(cmd, "mkdir \"%s\"", path);
				system(cmd);
				free(cmd);
			}
			tmp = tmp2 + 1;
		}
		
		path = realloc(path, strlen(path) + 1 + strlen(tmp) + 1);
		sprintf(path, "%s/%s", path, tmp);
		if(access(path, F_OK) != 0){
			char* cmd = malloc(8 + strlen(path) + 1);
			sprintf(cmd, "mkdir \"%s\"", path);
			system(cmd);
			free(cmd);
		}
		
		free(group);
		free(path);
		refreshViews();
	}
	
	gtk_widget_destroy(Entry);
	
}

void shortcuts_DeleteShortcut(void)
{
	GtkWidget* GameTreeView = glade_xml_get_widget(mainwindow, "GameTreeView");
	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(GameTreeView));
	
	int type = tree_GetTypeFromSelection(selection);
	
	char* path = tree_GetPathFromSelection(selection);
	if(path == NULL)
		return;
	
	gchar* warning;
	switch(type){
		case SHORTCUT_TYPE:
			warning = "Are you sure you want to permanently delete this shortcut?";
			break;
		case DIR_TYPE:
			warning = "Are you sure you want to delete this program group and all of its shortcuts?";
			break;
		case P2P_TYPE:
		case P2P_DIR_TYPE:
			show_Warning(
				"You cannot delete Point2Play shortcuts.  Use the preferences dialog to remove them from the tree.",
				glade_xml_get_widget(mainwindow, "MainWindow")
			);
			free(path);
			return;
	}
	
	GtkWidget* MainWindow = glade_xml_get_widget(mainwindow, "MainWindow");
	GtkWidget* Warning = gtk_message_dialog_new(
		GTK_WINDOW(MainWindow),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_YES_NO,
		warning);
	
	gint response = gtk_dialog_run(GTK_DIALOG(Warning));
	gtk_widget_destroy(Warning);
	if(response != GTK_RESPONSE_YES)
		return;
	
	if(type == SHORTCUT_TYPE) {
		unlink(path);
	
		char* favorite = strrchr(path, '/');
		*(strstr(favorite, ".desktop") + 8) = '\0';
		favorite = favorite + 1;
	
		char* favoritespath = malloc(strlen(getenv("HOME")) + 21 + strlen(favorite) + 1);
		sprintf(favoritespath, "%s/.grapevine/favorites/%s", getenv("HOME"), favorite);
		
		if(access(favoritespath, F_OK) == 0)
			unlink(favoritespath);
	} else if (type == DIR_TYPE) {
		char* cmd = malloc(9 + strlen(path) + 1);
		sprintf(cmd, "rm -rf \"%s\"", path);
		system(cmd);
		free(cmd);
	}
		
	refreshViews();
}	

void NewShortcutCustomDirButton_toggled(GtkWidget* CustomDirButton)
{
	GtkWidget* Entry;
	GtkWidget* BrowseButton;
	
	if(CustomDirButton == glade_xml_get_widget(shortcuts, "NewShortcutCustomDirButton")) {
		Entry = glade_xml_get_widget(shortcuts, "NewShortcutCustomDirEntry");
		BrowseButton = glade_xml_get_widget(shortcuts, "NewShortcutCustomDirBrowseButton");
	} else if(CustomDirButton == glade_xml_get_widget(shortcuts, "EditShortcutCustomDirButton")) {
		Entry = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirEntry");
		BrowseButton = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirBrowseButton");
	}

	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CustomDirButton));
	
	gtk_widget_set_sensitive(Entry, active);
	gtk_widget_set_sensitive(BrowseButton, active);
	
}

void BrowseButton_clicked(GtkWidget* Button)
{
	enum { EXEC, ICON };
	
	uint type;
	GtkWidget* Entry;
	const char* dir;
	
	if(Button == glade_xml_get_widget(shortcuts, "NewShortcutExecBrowseButton")) {
		type = EXEC;
		Entry = glade_xml_get_widget(shortcuts, "NewShortcutExecEntry");
		dir = getenv("HOME");
	}
	else if(Button == glade_xml_get_widget(shortcuts, "EditShortcutExecBrowseButton")) {
		type = EXEC;
		Entry = glade_xml_get_widget(shortcuts, "EditShortcutExecEntry");
		dir = getenv("HOME");
	}
	else if(Button == glade_xml_get_widget(shortcuts, "NewShortcutIconBrowseButton")) {
		type = ICON;
		Entry = glade_xml_get_widget(shortcuts, "NewShortcutIconEntry");
		dir = gtk_entry_get_text(GTK_ENTRY(Entry));
	}
	else if(Button == glade_xml_get_widget(shortcuts, "EditShortcutIconBrowseButton")) {
		type = ICON;
		Entry = glade_xml_get_widget(shortcuts, "EditShortcutIconEntry");
		dir = gtk_entry_get_text(GTK_ENTRY(Entry));
	}
		
	
	GtkWidget* FileBrowser = gtk_file_chooser_dialog_new ("",
				      NULL,
				      GTK_FILE_CHOOSER_ACTION_OPEN,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
				      NULL);
	
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(FileBrowser), dir);
	
	GtkFileFilter* allfiles = gtk_file_filter_new();
	gtk_file_filter_set_name(allfiles, "All Files");
	gtk_file_filter_add_pattern(allfiles, "*");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileBrowser), allfiles);
	
	GtkFileFilter* filter = gtk_file_filter_new();
	if(type == EXEC) {
		gtk_window_set_title(GTK_WINDOW(FileBrowser), "Select game executable");
		gtk_file_filter_set_name(filter, "EXE files");
		gtk_file_filter_add_pattern(filter, "*.exe");
	} 
	else if(type == ICON) {
		gtk_window_set_title(GTK_WINDOW(FileBrowser), "Select icon");
		gtk_file_filter_set_name(filter, "PNG and XPM files");
		gtk_file_filter_add_pattern(filter, "*.xpm");
		gtk_file_filter_add_pattern(filter, "*.png");
	}
	
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileBrowser), filter);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(FileBrowser), filter);
	
	gint response = gtk_dialog_run(GTK_DIALOG(FileBrowser));
	if(response != GTK_RESPONSE_OK){
		gtk_widget_destroy(FileBrowser);
		return;
	}
		
	gchar* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileBrowser));
	if(filename != NULL)
		gtk_entry_set_text(GTK_ENTRY(Entry), filename);
	
	g_free(filename);
	
	gtk_widget_destroy(FileBrowser);
}

void CustomDirBrowseButton_clicked(GtkWidget* Button)
{
	GtkWidget* Entry;
	if(Button == glade_xml_get_widget(shortcuts, "NewShortcutCustomDirBrowseButton"))
		Entry = glade_xml_get_widget(shortcuts, "NewShortcutCustomDirEntry");
	else if(Button == glade_xml_get_widget(shortcuts, "EditShortcutCustomDirBrowseButton"))
		Entry = glade_xml_get_widget(shortcuts, "EditShortcutCustomDirEntry");

	GtkWidget* FileBrowser = gtk_file_chooser_dialog_new(
		"Select custom working directory",
		NULL,
		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
		NULL);
	
	if(strcmp(gtk_entry_get_text(GTK_ENTRY(Entry)), "") == 0)
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(FileBrowser), getenv("HOME"));
	else
		gtk_file_chooser_set_current_folder(
			GTK_FILE_CHOOSER(FileBrowser),
			gtk_entry_get_text(GTK_ENTRY(Entry))
		);
	
	gint response = gtk_dialog_run(GTK_DIALOG(FileBrowser));
	if(response == GTK_RESPONSE_OK)
	{
		gchar* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileBrowser));
		if(filename != NULL)
			gtk_entry_set_text(GTK_ENTRY(Entry), filename);
		g_free(filename);
	}
	gtk_widget_destroy(FileBrowser);
}
