/***************************************************************************
 *            wineconfig.c
 *
 *  Thu Aug  5 20:39:19 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "confutils.h"
#include "wineconfig.h"
#include "winverutils.h"

uint VAL_TO_VAR(char* val)
{
	uint response;
	if(val == NULL)
		response = VAR_NULL;
	else if(val[0] == 'Y' || val[0] == 'y')
		response = 1;
	else if(val[0] == 'N' || val[0] == 'n')
		response = 0;
	else
		response = VAR_NULL;
	
	free(val);
	return response;
}

char* VAR_TO_VAL(uint var)
{
	if(var == 1)
		return "Y";
	else if(var == 0)
		return "N";
	else
		return "N";
}

uint PthreadsVar(char* val)
{
	if(val == NULL)
		return PTHREADS_AUTO;
	else if(val[0] == 'Y' || val[0] == 'y')
		return PTHREADS_YES;
	else if(val[0] == 'N' || val[0] == 'n')
		return PTHREADS_NO;
	
	return PTHREADS_AUTO;
}

void free_WineConfig(WineConfig* conf)
{
	uint i;
	for(i = 0; i < NUM_TEXT; i++)
		free(conf->text[i]);
}

void init_WineConfig(WineConfig* conf)
{
	uint i;
	for(i = 0; i < NUM_TEXT; i++)
		conf->text[i] = NULL;
}

void set_WineConfig_text(const char* text, uint NAME, WineConfig* conf)
{
	if(text != NULL){
		conf->text[NAME] = realloc(conf->text[NAME], strlen(text) + 1);
		strcpy(conf->text[NAME], text);
	} else {
		free(conf->text[NAME]);
		conf->text[NAME] = NULL;
	}
}

void wineconfig_SetGameConfig(WineConfig* conf, const char* executable)
{	
	// if the user selected "None" . . .
	if(strcmp(conf->text[DESKTOP_TEXT], "None") == 0)
		set_WineConfig_text(NULL, DESKTOP_TEXT, conf);
	
	conf_SetGameVal(conf->text[DESKTOP_TEXT], "Desktop", "x11drv", executable);
	conf_SetGameVal(conf->text[WORKDIR_TEXT], "doscwd", "transgaming", executable);
	conf_SetGameVal(conf->text[CMDLINE_TEXT], "cmdlineadd", "transgaming", executable);
	conf_SetGameVal(conf->text[WINEEXEC_TEXT], "winexversion", "transgaming", executable);
	
	// if the user selected "Default" . . .
	if(conf->vars[WINVER_VAR] == WINVER_DEFAULT)
		conf_SetGameVal(NULL, "Windows", "Version", executable);
	else
		conf_SetGameVal(WinverText(conf->vars[WINVER_VAR]), "Windows", "Version", executable);
	
	conf_SetGameVal( VAR_TO_VAL(conf->vars[MANAGED_VAR]), "Managed", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[CLIPSPACEFIX_VAR]), "ClipSpaceFix", "d3dgl", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[DXGRAB_VAR]), "DXGrab", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[XVIDMODE_VAR]), "UseXVidMode", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[XRANDR_VAR]), "UseXRandR", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[NVVAR_VAR]), "NV_VAR", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[ARBVBO_VAR]), "ARB_VBO", "x11drv", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[ANISO_VAR]), "AnisotropicTextureFiltering", "d3dgl", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[VSHADERS_VAR]), "VertexShaders", "d3dgl", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[PSHADERS_VAR]), "PixelShaders", "d3dgl", executable);
	conf_SetGameVal( VAR_TO_VAL(conf->vars[SHMWINESERVER_VAR]), "SHMWineserver", "Wineserver", executable);
	
	if(conf->vars[SOUNDDRV_VAR] == DRV_WINEALSA){
		conf_SetGameVal("winealsa.drv", "Drivers", "WinMM", executable);
		conf_SetGameVal( VAR_TO_VAL(conf->vars[MMAP_VAR]), "UseMMap", "winealsa", executable);
	} else if(conf->vars[SOUNDDRV_VAR] == DRV_WINEOSS){
		conf_SetGameVal("wineoss.drv", "Drivers", "WinMM", executable);
		conf_SetGameVal( VAR_TO_VAL(conf->vars[MMAP_VAR]), "UseMMap", "wineoss", executable);
		conf_SetGameVal( VAR_TO_VAL(conf->vars[FDUPLEX_VAR]), "FullDuplex", "wineoss", executable);
	}
	
	conf_SetGameVal( 
		VAR_TO_VAL(conf->vars[RENICEWINESERVER_VAR]), "renice_wineserver", "transgaming", executable);
	
	conf_SetGameVal( VAR_TO_VAL(conf->vars[BIGEXE_VAR]), "bigexe", "transgaming", executable);
	if(conf->vars[PTHREADS_VAR] == PTHREADS_AUTO)
		conf_SetGameVal("Auto", "force_pthreads", "transgaming", executable);
	else if(conf->vars[PTHREADS_VAR] == PTHREADS_YES)
		conf_SetGameVal("Y", "force_pthreads", "transgaming", executable);
	else if(conf->vars[PTHREADS_VAR] == PTHREADS_NO)
		conf_SetGameVal("N", "force_pthreads", "transgaming", executable);
	
}

gboolean wineconfig_GetGameConfig(WineConfig* conf, const char* executable)
{
	char filename[strlen(getenv("HOME")) + 20];
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
	
	FILE* ConfFile = fopen(filename, "r");
	if(ConfFile == NULL){
	 	printf("Error opening config file: %s", filename);
		return FALSE;
	}
	
	free_WineConfig(conf);
	init_WineConfig(conf);
	
	conf->text[DESKTOP_TEXT] = conf_GetGameVal("Desktop", "x11drv", executable, ConfFile);
	conf->text[WORKDIR_TEXT] = conf_GetGameVal("doscwd", "transgaming", executable, ConfFile);
	if(conf->text[WORKDIR_TEXT] != NULL)
		if(strcmp(conf->text[WORKDIR_TEXT], "") == 0)
			set_WineConfig_text(NULL, WORKDIR_TEXT, conf);
	
	conf->text[CMDLINE_TEXT] = conf_GetGameVal("cmdlineadd", "transgaming", executable, ConfFile);
	if(conf->text[CMDLINE_TEXT] != NULL)
		if(strcmp(conf->text[CMDLINE_TEXT], "") == 0)
			set_WineConfig_text(NULL, CMDLINE_TEXT, conf);
		
	conf->text[WINEEXEC_TEXT] = conf_GetGameVal("winexversion", "transgaming", executable, ConfFile);
	if(conf->text[WINEEXEC_TEXT] != NULL)
		if(strcmp(conf->text[WINEEXEC_TEXT], "") == 0)
			set_WineConfig_text(NULL, WINEEXEC_TEXT, conf);
	
	conf->vars[WINVER_VAR]   = WinverIndex(conf_GetCustomVal("Windows", "Version", executable, ConfFile));
	conf->vars[MANAGED_VAR]  = VAL_TO_VAR(conf_GetGameVal("Managed", "x11drv", executable, ConfFile));
	conf->vars[CLIPSPACEFIX_VAR]  = VAL_TO_VAR(conf_GetGameVal("ClipSpaceFix", "d3dgl", executable, ConfFile));
	conf->vars[DXGRAB_VAR]   = VAL_TO_VAR(conf_GetGameVal("DXGrab", "x11drv", executable, ConfFile));
	conf->vars[XVIDMODE_VAR] = VAL_TO_VAR(conf_GetGameVal("UseXVidMode", "x11drv", executable, ConfFile));
	conf->vars[XRANDR_VAR]   = VAL_TO_VAR(conf_GetGameVal("UseXRandR", "x11drv", executable, ConfFile));
	conf->vars[NVVAR_VAR]    = VAL_TO_VAR(conf_GetGameVal("NV_VAR", "x11drv", executable, ConfFile));
	conf->vars[ARBVBO_VAR]   = VAL_TO_VAR(conf_GetGameVal("ARB_VBO", "x11drv", executable, ConfFile));
	conf->vars[ANISO_VAR]    =
		VAL_TO_VAR(conf_GetGameVal("AnisotropicTextureFiltering", "d3dgl", executable, ConfFile));
	conf->vars[VSHADERS_VAR] = VAL_TO_VAR(conf_GetGameVal("VertexShaders", "d3dgl", executable, ConfFile));
	conf->vars[PSHADERS_VAR] = VAL_TO_VAR(conf_GetGameVal("PixelShaders", "d3dgl", executable, ConfFile));
	
	char* tmp = conf_GetGameVal("Drivers", "WinMM", executable, ConfFile);
	if(tmp == NULL){
		conf->vars[SOUNDDRV_VAR]  = VAR_NULL;
		conf->vars[MMAP_VAR]      = VAR_NULL;
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	} else if(strcmp(tmp, "wineoss.drv") == 0) {
		conf->vars[SOUNDDRV_VAR]  = DRV_WINEOSS;
		conf->vars[MMAP_VAR]      = VAL_TO_VAR(conf_GetGameVal("UseMMap", "wineoss", executable, ConfFile));
		conf->vars[FDUPLEX_VAR]	  =
			VAL_TO_VAR(conf_GetGameVal("FullDuplex", "wineoss", executable, ConfFile));
	} else if(strcmp(tmp, "winealsa.drv") == 0) {
		conf->vars[SOUNDDRV_VAR]  = DRV_WINEALSA;
		conf->vars[MMAP_VAR]      = VAL_TO_VAR(conf_GetGameVal("UseMMap", "winealsa", executable, ConfFile));
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	} else {
		conf->vars[SOUNDDRV_VAR]  = VAR_NULL;
		conf->vars[MMAP_VAR]      = VAR_NULL;
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	}
	if(tmp != NULL)
		free(tmp);
	
	conf->vars[FREETYPE_VAR] = VAL_TO_VAR(conf_GetGameVal("FreeType", "fonts", executable, ConfFile));
	
	conf->vars[SHMWINESERVER_VAR] = 
		VAL_TO_VAR(conf_GetGameVal("SHMWineserver", "Wineserver", executable, ConfFile));
	
	conf->vars[RENICEWINESERVER_VAR] = 
		VAL_TO_VAR(conf_GetGameVal("renice_wineserver", "transgaming", executable, ConfFile));
	
	conf->vars[BIGEXE_VAR]   = VAL_TO_VAR(conf_GetGameVal("bigexe", "transgaming", executable, ConfFile));
	conf->vars[PTHREADS_VAR] =
		PthreadsVar(conf_GetGameVal("force_pthreads", "transgaming", executable, ConfFile));
	
	fclose(ConfFile);
	return TRUE;
	
}

void wineconfig_SetGlobalConfig(WineConfig* conf)
{	
	// if the user selected "None" . . .
	if(strcmp(conf->text[DESKTOP_TEXT], "None") == 0)
		set_WineConfig_text(NULL, DESKTOP_TEXT, conf);
	
	conf_SetVal(conf->text[DESKTOP_TEXT], "Desktop", "x11drv");
	
	conf_SetVal(conf->text[WINEEXEC_TEXT], "winexversion", "transgaming");
	
	// if the user selected "Default" . . .
	if(conf->vars[WINVER_VAR] == WINVER_DEFAULT)
		conf_SetVal(NULL, "Windows", "Version");
	else
		conf_SetVal(WinverText(conf->vars[WINVER_VAR]), "Windows", "Version");
	
	conf_SetVal( VAR_TO_VAL(conf->vars[MANAGED_VAR]), "Managed", "x11drv");
	conf_SetVal( VAR_TO_VAL(conf->vars[CLIPSPACEFIX_VAR]), "ClipSpaceFix", "d3dgl");
	conf_SetVal( VAR_TO_VAL(conf->vars[DXGRAB_VAR]), "DXGrab", "x11drv");
	conf_SetVal( VAR_TO_VAL(conf->vars[XVIDMODE_VAR]), "UseXVidMode", "x11drv");
	conf_SetVal( VAR_TO_VAL(conf->vars[XRANDR_VAR]), "UseXRandR", "x11drv");
	
	char* tmp = malloc(5);
	g_ascii_dtostr(tmp, 5, conf->vars[VIDEORAM_VAR]);
	conf_SetVal(tmp, "VideoRam", "x11drv");
	g_ascii_dtostr(tmp, 5, conf->vars[AGPMEM_VAR]);
	conf_SetVal(tmp, "AGPVertexRam", "x11drv");
	free(tmp);
	
	conf_SetVal( VAR_TO_VAL(conf->vars[NVVAR_VAR]), "NV_VAR", "x11drv");
	conf_SetVal( VAR_TO_VAL(conf->vars[ARBVBO_VAR]), "ARB_VBO", "x11drv");
	conf_SetVal( VAR_TO_VAL(conf->vars[ANISO_VAR]), "AnisotropicTextureFiltering", "d3dgl");
	conf_SetVal( VAR_TO_VAL(conf->vars[VSHADERS_VAR]), "VertexShaders", "d3dgl");
	conf_SetVal( VAR_TO_VAL(conf->vars[PSHADERS_VAR]), "PixelShaders", "d3dgl");
	
	if(conf->vars[SOUNDDRV_VAR] == DRV_WINEALSA){
		conf_SetVal("winealsa.drv", "Drivers", "WinMM");
		conf_SetVal( VAR_TO_VAL(conf->vars[MMAP_VAR]), "UseMMap", "winealsa");
	} else if(conf->vars[SOUNDDRV_VAR] == DRV_WINEOSS){
		conf_SetVal("wineoss.drv", "Drivers", "WinMM");
		conf_SetVal( VAR_TO_VAL(conf->vars[MMAP_VAR]), "UseMMap", "wineoss");
		conf_SetVal( VAR_TO_VAL(conf->vars[FDUPLEX_VAR]), "FullDuplex", "wineoss");
	}
	
	if(strcmp(conf->text[ALSA_DEV_TEXT], "!AUTO") == 0) {
		conf_RemoveVal("pcm0", "winealsa");
		conf_RemoveVal("ctl0", "winealsa");
	} else {
		conf_SetVal(conf->text[ALSA_DEV_TEXT], "pcm0", "winealsa");
		conf_SetVal(conf->text[ALSA_CTL_TEXT], "ctl0", "winealsa");
	}
	
	if(strcmp(conf->text[OSS_DEV_TEXT], "!AUTO") == 0) {
		conf_RemoveVal("dsp0", "wineoss");
		conf_RemoveVal("mixer0", "wineoss");
	} else {
		conf_SetVal(conf->text[OSS_DEV_TEXT], "dsp0", "wineoss");
		conf_SetVal(conf->text[OSS_MXR_TEXT], "mixer0", "wineoss");
	}
	
	conf_SetVal( VAR_TO_VAL(conf->vars[FREETYPE_VAR]), "FreeType", "fonts");
	conf_SetVal( VAR_TO_VAL(conf->vars[SHMWINESERVER_VAR]), "SHMWineserver", "Wineserver");
	
	conf_SetVal( 
		VAR_TO_VAL(conf->vars[RENICEWINESERVER_VAR]), "renice_wineserver", "transgaming");
	
	if(conf->vars[PTHREADS_VAR] == PTHREADS_AUTO)
		conf_SetVal("Auto", "force_pthreads", "transgaming");
	else if(conf->vars[PTHREADS_VAR] == PTHREADS_YES)
		conf_SetVal("Y", "force_pthreads", "transgaming");
	else if(conf->vars[PTHREADS_VAR] == PTHREADS_NO)
		conf_SetVal("N", "force_pthreads", "transgaming");
}

gboolean wineconfig_GetGlobalConfig(WineConfig* conf)
{
	char filename[strlen(getenv("HOME")) + 20];
	sprintf(filename, "%s/.transgaming/config", getenv("HOME"));
	
	FILE* ConfFile = fopen(filename, "r");
	if(ConfFile == NULL){
	 	printf("Error opening config file: %s", filename);
		return FALSE;
	}
	
	free_WineConfig(conf);
	init_WineConfig(conf);
	
	conf->text[DESKTOP_TEXT]  = conf_GetVal("Desktop", "x11drv", ConfFile);
	conf->text[ALSA_DEV_TEXT] = conf_GetVal("pcm0", "winealsa", ConfFile);
	conf->text[ALSA_CTL_TEXT] = conf_GetVal("ctl0", "winealsa", ConfFile);
	conf->text[WINEEXEC_TEXT] = conf_GetVal("winexversion", "transgaming", ConfFile);
	
	char* tmp = conf_GetVal("Windows", "Version", ConfFile);
	conf->vars[WINVER_VAR] = WinverIndex(tmp);
	free(tmp);
	
	conf->vars[MANAGED_VAR]  = VAL_TO_VAR(conf_GetVal("Managed", "x11drv", ConfFile));
	conf->vars[CLIPSPACEFIX_VAR]  = VAL_TO_VAR(conf_GetVal("ClipSpaceFix", "d3dgl", ConfFile));
	conf->vars[DXGRAB_VAR]   = VAL_TO_VAR(conf_GetVal("DXGrab", "x11drv", ConfFile));
	conf->vars[XVIDMODE_VAR] = VAL_TO_VAR(conf_GetVal("UseXVidMode", "x11drv", ConfFile));
	conf->vars[XRANDR_VAR]   = VAL_TO_VAR(conf_GetVal("UseXRandR", "x11drv", ConfFile));
	
	conf->vars[VIDEORAM_VAR] = g_strtod(conf_GetVal("VideoRam", "x11drv", ConfFile), NULL);
	conf->vars[AGPMEM_VAR]   = g_strtod(conf_GetVal("AGPVertexRam", "x11drv", ConfFile), NULL);
	conf->vars[NVVAR_VAR]    = VAL_TO_VAR(conf_GetVal("NV_VAR", "x11drv", ConfFile));
	conf->vars[ARBVBO_VAR]   = VAL_TO_VAR(conf_GetVal("ARB_VBO", "x11drv", ConfFile));
	
	conf->vars[ANISO_VAR]    = VAL_TO_VAR(conf_GetVal("AnisotropicTextureFiltering", "d3dgl",
		ConfFile));
	conf->vars[VSHADERS_VAR] = VAL_TO_VAR(conf_GetVal("VertexShaders", "d3dgl", ConfFile));
	conf->vars[PSHADERS_VAR] = VAL_TO_VAR(conf_GetVal("PixelShaders", "d3dgl", ConfFile));
	
	tmp = conf_GetVal("Drivers", "WinMM", ConfFile);
	if(tmp == NULL){
		conf->vars[SOUNDDRV_VAR]  = VAR_NULL;
		conf->vars[MMAP_VAR]      = VAR_NULL;
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	} else if(strcmp(tmp, "wineoss.drv") == 0) {
		conf->vars[SOUNDDRV_VAR]  = DRV_WINEOSS;
		conf->vars[MMAP_VAR]      = VAL_TO_VAR(conf_GetVal("UseMMap", "wineoss", ConfFile));
		conf->vars[FDUPLEX_VAR]   = VAL_TO_VAR(conf_GetVal("FullDuplex", "wineoss", ConfFile));
	} else if(strcmp(tmp, "winealsa.drv") == 0) {
		conf->vars[SOUNDDRV_VAR]  = DRV_WINEALSA;
		conf->vars[MMAP_VAR]      = VAL_TO_VAR(conf_GetVal("UseMMap", "winealsa",  ConfFile));
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	} else {
		conf->vars[SOUNDDRV_VAR]  = VAR_NULL;
		conf->vars[MMAP_VAR]      = VAR_NULL;
		conf->vars[FDUPLEX_VAR]   = VAR_NULL;
	}
	if(tmp != NULL)
		free(tmp);
	
	conf->vars[FREETYPE_VAR] = VAL_TO_VAR(conf_GetVal("FreeType", "fonts", ConfFile));
	conf->vars[SHMWINESERVER_VAR] = 
		VAL_TO_VAR(conf_GetVal("SHMWineserver", "Wineserver", ConfFile));
	
	conf->vars[RENICEWINESERVER_VAR] = 
		VAL_TO_VAR(conf_GetVal("renice_wineserver", "transgaming", ConfFile));
	
	tmp = conf_GetVal("force_pthreads", "transgaming", ConfFile);
	if(tmp == NULL)
		conf->vars[PTHREADS_VAR] = PTHREADS_AUTO;
	else if(strcmp(tmp, "Y") == 0)
		conf->vars[PTHREADS_VAR] = PTHREADS_YES;
	else if(strcmp(tmp, "N") == 0)
		conf->vars[PTHREADS_VAR] = PTHREADS_NO;
	else if(strcmp(tmp, "Auto") == 0)
		conf->vars[PTHREADS_VAR] = PTHREADS_AUTO;
	free(tmp);
	
	fclose(ConfFile);
	return TRUE;
	
}
