/***************************************************************************
 *            wineconfig.h
 *
 *  Thu Aug  5 20:44:46 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _WINECONFIG_H
#define _WINECONFIG_H

#include <stdlib.h>
#include <glib.h>

#define VAR_NULL 0

// WineConfig->text types
enum {
	DESKTOP_TEXT,
	ALSA_DEV_TEXT,
	ALSA_CTL_TEXT,
	OSS_DEV_TEXT,
	OSS_MXR_TEXT,
	WORKDIR_TEXT,
	CMDLINE_TEXT,
	WINEEXEC_TEXT,
	NUM_TEXT
};

// WineConfig->vars types
enum {
	WINVER_VAR,
	MANAGED_VAR,
	CLIPSPACEFIX_VAR,
	DXGRAB_VAR,
	XVIDMODE_VAR,
	XRANDR_VAR,
	VIDEORAM_VAR,
	AGPMEM_VAR,
	NVVAR_VAR,
	ARBVBO_VAR,
	ANISO_VAR,
	VSHADERS_VAR,
	PSHADERS_VAR,
	SOUNDDRV_VAR,
	MMAP_VAR,
	FDUPLEX_VAR,
	FREETYPE_VAR,
	SHMWINESERVER_VAR,
	RENICEWINESERVER_VAR,
	BIGEXE_VAR,
	PTHREADS_VAR,
	NUM_VAR,
};

// SOUNDDRV_VAR types (with space reserved for VAR_NULL)
enum {
	none,
	DRV_WINEOSS,
	DRV_WINEALSA
};

// PTHREADS_VAR types (conveniently, Auto and VAR_NULL serve the same purpose)
enum {
	PTHREADS_AUTO,
	PTHREADS_YES,
	PTHREADS_NO
};

struct WineConfig {
	char* text[NUM_TEXT];
	uint vars[NUM_VAR];
};
typedef struct WineConfig WineConfig;

// converts from "Y" or "N" to 1 or 0
uint VAL_TO_VAR(char* val);

// converts from 1 or 0 to "Y" or "N"
char* VAR_TO_VAL(uint var);

// return the variable for Y or N
uint PthreadsVar(char* val);

// should be called after we're done using a WineConfig
void free_WineConfig(WineConfig* conf);

// should be called after creating a WineConfig
void init_WineConfig(WineConfig* conf);

// correct method for resetting a WineConfig->text field (dynamically reallocates)
void set_WineConfig_text(const char* text, uint name, WineConfig* conf);

// put config for a game into conf
gboolean wineconfig_GetGameConfig(WineConfig* conf, const char* executable);

// set config for a game based on conf
void wineconfig_SetGameConfig(WineConfig* conf, const char* executable);

// put global config into conf
gboolean wineconfig_GetGlobalConfig(WineConfig* conf);

// set global config based on conf
void wineconfig_SetGlobalConfig(WineConfig* conf);

#endif
